from Acquisition import aq_inner
from plone import api
from Products.Five.browser import BrowserView


def redirect_to_parent(context):
    context = aq_inner(context).aq_explicit
    # Auto redirect to the anchor
    param = '#{0}'.format(context.id)
    return context.REQUEST.RESPONSE.redirect(
        context.aq_parent.absolute_url() + param)


class RedirectToParent(BrowserView):

    def __call__(self):
        return redirect_to_parent(self.context)


class BlockContainerRedirectToParent(RedirectToParent):

    def __call__(self):
        if api.user.is_anonymous():
            return super(BlockContainerRedirectToParent, self).__call__()
        else:
            context = aq_inner(self.context).aq_explicit
            return self.request.RESPONSE.redirect(
                context.absolute_url() + '/folder_contents')



from collective.geo.behaviour import MessageFactory as CGMF
from ftw.simplelayout import _
from ftw.simplelayout.mapblock.contents.interfaces import IMapBlock
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.content import Item
from plone.directives import form
from plone.supermodel import model
from zope import schema
from zope.interface import alsoProvides
from zope.interface import implements


class IMapBlockSchema(form.Schema):
    """MapBlock for simplelayout
    """

    title = schema.TextLine(
        title=_(u'label_title', default=u'Title'),
        required=False)

    show_title = schema.Bool(
        title=_(u'label_show_title', default=u'Show title'),
        default=True,
        required=False)

    form.mode(zoomlevel='hidden')
    zoomlevel = schema.TextLine(
        title=u"Zoom",
        required=False,
    )

    form.mode(maplayer='hidden')
    maplayer = schema.TextLine(
        title=u"Maplayer",
        required=False,
    )

    model.fieldset(
        'coordinates',
        label=CGMF(u'Coordinates'),
        fields=('zoomlevel', 'maplayer', 'title', 'show_title')
    )


alsoProvides(IMapBlockSchema, IFormFieldProvider)


class MapBlock(Item):
    implements(IMapBlock)



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from plone.app.textfield.value import RichTextValue
import transaction


class TestSearchableTextIndexer(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.setup_sample_ftis(self.portal)
        self.setup_block_views()
        transaction.commit()

        self.contentpage = create(Builder('sample container')
                                  .titled(u'ContentPage'))
        self.textblock = create(Builder('sample block')
                                .titled(u'TextBlock')
                                .within(self.contentpage)
                                .having(text=RichTextValue(u'asdf')))

    def search_for(self, term, path=None):
        query = {'SearchableText': term,
                 'portal_type': 'SampleContainer'}
        if path:
            query['path'] = path
        return self.layer['portal'].portal_catalog(query)

    def test_searchable_text_is_indexed_on_container(self):
        result = self.search_for('asdf')
        assert len(result) == 1, 'Expect exactly one brain'
        self.assertEquals(self.contentpage,
                          result[0].getObject(),
                          'Expect the contentpage which has a textblock with'
                          ' the text "Text"')

    def test_searchable_text_is_up_to_date_on_delete(self):
        self.contentpage.manage_delObjects(['textblock'])
        result = self.search_for('asdf')
        self.assertEquals(0, len(result), 'Expect no entry')

    def test_searchable_text_is_up_to_date_on_move(self):
        second_page = create(Builder('sample container')
                             .titled(u'ContentPage2'))

        cut = self.contentpage.manage_cutObjects([self.textblock.getId()])
        second_page.manage_pasteObjects(cut)

        result = self.search_for('asdf')
        self.assertEquals(result[0].getURL(), second_page.absolute_url())

    def test_searchable_text_is_up_to_date_on_copy(self):
        second_page = create(Builder('sample container')
                             .titled(u'ContentPage2'))

        copy = self.contentpage.manage_copyObjects([self.textblock.getId()])
        second_page.manage_pasteObjects(copy)

        result = self.search_for('asdf')
        self.assertEquals(2, len(result), 'Expect 2 brains.')



from ftw.simplelayout.contenttypes.contents.interfaces import IContentPage
from plone.dexterity.content import Container
from plone.supermodel import model
from zope.interface import implements


class IContentPageSchema(model.Schema):
    """A Folderish page type for blocks
    """


class ContentPage(Container):
    implements(IContentPage)



from ftw.simplelayout.utils import IS_PLONE_5
from plone.registry.interfaces import IRegistry
from zope.component import getUtility


def profile_uninstalled(site):
    if IS_PLONE_5:
        clean_plone5_registry(site)


def clean_plone5_registry(site):
    registry = getUtility(IRegistry)

    types_not_searched = list(registry['plone.types_not_searched'])
    if 'ftw.simplelayout.MapBlock' in types_not_searched:
        types_not_searched.remove('ftw.simplelayout.MapBlock')
        registry['plone.types_not_searched'] = tuple(types_not_searched)



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.interfaces import IBlockProperties
from ftw.simplelayout.properties import MultiViewBlockProperties
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_INTEGRATION_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from zope.component import getMultiAdapter
from zope.interface.verify import verifyClass


class TestMultiViewBlockProperties(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_INTEGRATION_TESTING

    def setUp(self):
        self.setup_sample_ftis(self.layer['portal'])
        contentpage = create(Builder('sample container'))
        self.setup_block_views()
        self.block = create(Builder('sample block').within(contentpage))

    def test_implements_interface(self):
        verifyClass(IBlockProperties, MultiViewBlockProperties)

    def test_get_current_default_block_view(self):
        properties = getMultiAdapter((self.block, self.block.REQUEST),
                                     IBlockProperties)

        self.assertEqual(properties.get_current_view_name(),
                         'block_view')

    def test_changing_view(self):
        properties = getMultiAdapter((self.block, self.block.REQUEST),
                                     IBlockProperties)

        properties.set_view('block_view_different')
        self.assertEqual(properties.get_current_view_name(),
                         'block_view_different')

    def test_changing_view_unicode(self):
        properties = getMultiAdapter((self.block, self.block.REQUEST),
                                     IBlockProperties)

        properties.set_view(u'block_view_different')
        self.assertEqual(properties.get_current_view_name(),
                         'block_view_different')

    def test_setting_invalid_view_name_raises_error(self):
        properties = getMultiAdapter((self.block, self.block.REQUEST),
                                     IBlockProperties)

        with self.assertRaises(ValueError) as cm:
            properties.set_view('unkown-view')
        self.assertEqual(str(cm.exception),
                         '"unkown-view" is not in available views.')



from ftw.simplelayout.browser.ajax.utils import json_response
from ftw.simplelayout.interfaces import ISimplelayout
from ftw.simplelayout.utils import get_block_html
from plone.app.uuid.utils import uuidToObject
from plone.dexterity.browser.add import DefaultAddForm, DefaultAddView
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import BoundPageTemplate
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage
from zope.component import adapts
from zope.interface import implements
from zope.interface import Interface
from zope.traversing.interfaces import ITraversable
from zope.traversing.interfaces import TraversalError


class AddViewTraverser(object):
    """Add view traverser.
    """
    adapts(ISimplelayout, Interface)
    implements(ITraversable)

    template = ViewPageTemplateFile('templates/add.pt')

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def traverse(self, name, ignored):
        ttool = getToolByName(self.context, 'portal_types')
        ti = ttool.getTypeInfo(name)
        if ti is not None:
            add_view = AddView(self.context, self.request, ti)

            if add_view is not None:
                add_view.__name__ = ti.factory
                add_view.index = BoundPageTemplate(self.template.im_func,
                                                   add_view)
                return add_view.__of__(self.context)

        raise TraversalError(self.context, name)


class AddForm(DefaultAddForm):

    def createAndAdd(self, data):
        obj = super(AddForm, self).createAndAdd(data)
        self.obj_uid = obj.UID()
        aq_obj = obj.__of__(self.context)
        self.obj_html = get_block_html(aq_obj)
        return obj

    def render(self):
        if self._finishedAdd:
            return json_response(self.request, proceed=True)
        return super(AddForm, self).render()


class AddView(DefaultAddView):
    form = AddForm

    def render(self):
        if hasattr(self.form_instance, 'obj_uid'):
            obj = uuidToObject(self.form_instance.obj_uid)
            self.request.response.setHeader('Content-Type', 'application/json')

            # Consume all statusmessages
            IStatusMessage(self.request).show()

            return json_response(self.request,
                                 uid=self.form_instance.obj_uid,
                                 url=obj.absolute_url(),
                                 content=self.form_instance.obj_html,
                                 proceed=True,
                                 id=obj.id)

        return json_response(self.request,
                             content=super(AddView, self).render(),
                             proceed=False)



from ftw.simplelayout.contenttypes.contents.interfaces import IGalleryBlock
from ftw.simplelayout.contenttypes.contents.interfaces import ITextBlock
from ftw.simplelayout.handlers import unwrap_persistence
from ftw.simplelayout.interfaces import IPageConfiguration
from plone.app.uuid.utils import uuidToObject
from Products.Five.browser import BrowserView
import json
import re


class LeadImageView(BrowserView):
    """Returns the first image of a textblock with image placed in the
    'default' simplelayout container."""

    _has_image = None
    block = None

    def __call__(self, scale=None, direction='down'):
        if self.has_image:
            scale = self.get_scale(scale, direction=direction)
            return scale.tag()
        else:
            return ''

    @property
    def has_image(self):
        self._load()
        return self._has_image

    def get_scale(self, scale=None, direction='down'):
        self._load()
        if not self.has_image:
            return

        scale = scale or self.request.get('scale', 'preview')
        scaler = self.block.restrictedTraverse('@@images')
        return scaler.scale('image', scale=scale, direction=direction)

    def _get_uids(self):
        page_conf = IPageConfiguration(self.context)
        # look for blocks in default slot first ...
        state_string = json.dumps(unwrap_persistence(
            page_conf.load().get('default', {})))
        # ... then look in all slots.
        state_string += json.dumps(unwrap_persistence(
            page_conf.load()))
        return re.findall(r'\"uid\"\: \"(.+?)\"', state_string)

    def _load(self):
        if getattr(self, '_loaded', False):
            return
        self._loaded = True

        for uid in self._get_uids():
            block = uuidToObject(uid)
            if ITextBlock.providedBy(block) and block.image \
               and block.image.data:
                self._has_image = True
                self.block = block
                return

            if IGalleryBlock.providedBy(block):
                for image in block.listFolderContents({'portal_type': 'Image'}):
                    self._has_image = True
                    self.block = image
                    return



from ftw.upgrade import UpgradeStep


class RegisterRequestTrackerJavascript(UpgradeStep):
    """Register RequestTracker javascript.
    """

    def __call__(self):
        self.install_upgrade_profile()



from ftw.upgrade import UpgradeStep


class ReplaceMablockCoorinatesBehavior(UpgradeStep):
    """Replace mablock coorinates behavior.
    """

    def __call__(self):
        self.install_upgrade_profile()



from copy import deepcopy
from ftw.simplelayout.interfaces import IBlockConfiguration
from ftw.simplelayout.interfaces import IPageConfiguration
from ftw.simplelayout.interfaces import ISimplelayoutBlock
from ftw.simplelayout.interfaces import ISimplelayoutContainerConfig
from operator import itemgetter
from operator import methodcaller
from persistent.list import PersistentList
from persistent.mapping import PersistentMapping
from plone import api
from plone.uuid.interfaces import IUUID
from Products.CMFPlone.events import ReorderedEvent
from zExceptions import Unauthorized
from zope.annotation import IAnnotations
from zope.component import queryMultiAdapter
from zope.component.hooks import getSite
from zope.event import notify
from zope.interface import implements


SL_ANNOTATION_KEY = 'ftw.simplelayout.pageconfiguration'
BLOCK_ANNOTATION_KEY = 'ftw.simplelayout.blockconfiguration'


def make_resursive_persistent(conf):
    """Convert the given layout configuration to a rows/columns/blocks
       mapping.
    """
    # TODO: validate data

    def persist(data):
        if isinstance(data, dict):
            data = PersistentMapping(data)

            for key, value in data.items():
                data[key] = persist(value)

        elif isinstance(data, list):
            return PersistentList(map(persist, data))

        else:
            # Usually we got basestrings, or integer here, so do nothing.
            pass

        return data

    return persist(conf)


def columns_in_config(config, container=None):
    """Returns each column-dict found in the config.
    """
    columns = []

    if container is None:
        containers = config.values()
    else:
        containers = [config.get(container, [{"cols": [{"blocks": []}]}])]

    for container in containers:
        for layout in container:
            columns.extend(layout.get('cols', ()))

    return columns


def flattened_block_uids(config):
    """Accepting a page configuration, this function returns a flat list of block
    uids found in the configuration in order of appearance.
    """

    uids = []

    for column in columns_in_config(config):
        for block in column.get('blocks', ()):
            uids.append(block.get('uid'))

    return uids


def block_uids_in_page(page):
    """Returns a list of block UIDs of a page.
    """
    return map(IUUID, filter(ISimplelayoutBlock.providedBy, page.objectValues()))


def block_uids_missing_in_config(page):
    """Returns a list of UIDs of blocks, which are missing in the page's config.
    """

    in_config = flattened_block_uids(IPageConfiguration(page).load())
    return filter(lambda uid: uid not in in_config, block_uids_in_page(page))


def synchronize_page_config_with_blocks(page):
    """Updates the page config:
    - adds blocks to the config, which exist in the page but not in the config
    - removes blocks from the config, which do no longer exist in the page

    A dict with ``added`` and ``removed`` block UIDs is returned.
    """

    config = IPageConfiguration(page).load()
    columns = columns_in_config(config)
    existing_uids = block_uids_in_page(page)
    removed = []

    for column in columns:
        new_blocks = []
        for block in column.get('blocks', ()):
            if block['uid'] in existing_uids:
                new_blocks.append(block)
            else:
                removed.append(block['uid'])

        column['blocks'][:] = new_blocks

    # Add missing blocks to the bottom of the first column of the "default"
    # container.
    added = block_uids_missing_in_config(page)
    for uid in added:
        default_columns = columns_in_config(config, container='default')
        default_columns[0]['blocks'].append({'uid': uid})

    IPageConfiguration(page).store(config)
    return {'added': added, 'removed': removed}


def has_slot_layout_changed(old_data, new_data):
    old_layouts = map(len, map(itemgetter('cols'), old_data))
    new_layouts = map(len, map(itemgetter('cols'), new_data))
    return old_layouts != new_layouts


class PageConfiguration(object):
    """Adapter for storing simplelayout page configuration.
    """
    implements(IPageConfiguration)

    def __init__(self, context):
        self.context = context

    def store(self, conf, update_positions=True):
        self.check_permission(conf)
        annotations = IAnnotations(self.context)

        storage = annotations.get(SL_ANNOTATION_KEY, None)
        if storage:
            storage.update(make_resursive_persistent(conf))
        else:
            annotations[SL_ANNOTATION_KEY] = make_resursive_persistent(conf)

        if update_positions:
            self.update_object_positions()

    def load(self):
        annotations = IAnnotations(self.context)
        default_state = deepcopy(self._default_page_config())
        page_state = deepcopy(annotations.get(
            SL_ANNOTATION_KEY,
            make_resursive_persistent(default_state)))

        for slotname in default_state:
            if not page_state.get(slotname):
                page_state[slotname] = default_state[slotname]

        return page_state

    def check_permission(self, new_state):
        if api.user.has_permission('ftw.simplelayout: Change Layouts',
                                   obj=self.context):
            return

        # The user does not have the permission to manipulate the
        # structure. But the new configuration is serialized from the DOM
        # So we need to check if the user has manipulated the DOM by
        # himself. This is done by checking the stored state and the new state
        # generated from the DOM.

        old_state = self.load()

        for slot_name, new_slot_state in new_state.items():
            if slot_name not in old_state:
                continue

            if has_slot_layout_changed(old_state[slot_name], new_slot_state):
                raise Unauthorized()

    def get_ordered_blocks(self):
        """Returns all blocks in the page, ordered by the order stored
        in the page state.
        """
        uid_order = flattened_block_uids(self.load())

        def block_sort_key(block):
            uid = IUUID(block)
            if uid not in uid_order:
                return '~'
            else:
                return uid_order.index(uid)

        blocks = filter(ISimplelayoutBlock.providedBy,
                        map(self.context._getOb,
                            self.context.objectIds()))
        blocks.sort(key=block_sort_key)
        return blocks

    def update_object_positions(self):
        """Update the positions of the sl blocks in the orderable folder.
        """
        block_ids = map(methodcaller('getId'), self.get_ordered_blocks())
        self.context.moveObjectsToTop(block_ids)

        notify(ReorderedEvent(self.context))  # ftw.gopip support

    def _default_page_config(self):
        """Returns a default page config"""
        request = getattr(self.context, 'REQUEST', getSite().REQUEST)
        adapter = queryMultiAdapter((self.context, request),
                                    ISimplelayoutContainerConfig)

        if adapter is not None:
            layout = adapter.default_page_layout()
            if layout is not None:
                return layout

        return {'default': [{"cols": [{"blocks": []}]}]}


class BlockConfiguration(object):
    """Adapter for storing block configuration.
    """
    implements(IBlockConfiguration)

    def __init__(self, context):
        self.context = context

    def store(self, data):
        annotations = IAnnotations(self.context)
        annotations[BLOCK_ANNOTATION_KEY] = PersistentMapping(data)

    def load(self):
        annotations = IAnnotations(self.context)
        return annotations.get(BLOCK_ANNOTATION_KEY,
                               PersistentMapping())



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.contenttypes.behaviors import ITeaser
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.testbrowser import browsing
from unittest import TestCase
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.intid.interfaces import IIntIds
import transaction


class TestTeaserBehavior(TestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def setUp(self):
        self.page = create(Builder('sl content page'))
        self.intids = getUtility(IIntIds)

    def test_textblock_has_internal_link_and_external_link_field(self):
        textblock = create(Builder('sl textblock')
                           .within(self.page)
                           .having(external_link='http://www.4teamwork.ch')
                           .having(internal_link=RelationValue(
                                   self.intids.getId(self.page))))

        self.assertEquals('http://www.4teamwork.ch',
                          ITeaser(textblock).external_link)
        self.assertEquals(self.page,
                          ITeaser(textblock).internal_link.to_object)

    @browsing
    def test_broken_internal_link(self, browser):
        target = create(Builder('sl content page'))
        textblock = create(Builder('sl textblock')
                           .within(self.page)
                           .having(internal_link=RelationValue(
                                   self.intids.getId(target))))

        self.layer['portal'].manage_delObjects(ids=[target.getId()])
        transaction.commit()

        browser.login().visit(textblock, view='block_view')
        self.assertFalse(
            browser.css('a'),
            'Expect no link, since the internal target does not exist.')



from zope.interface import Interface


class IOpenGraphSupport(Interface):
    """Marker Interface for Opengraphable content"""


class IOpenGraphDataProvider(Interface):
    """Data provider for opengraphable content
        By default it needs to provide the basic metadata fields:
        - og:title
        - og:type
        - og:image
        - og:url

        I a content is part of larger website you can also provide:
        - og:site:name

        You can provide more fields as you wish, check http://ogp.me

        We use a OrderedDict, since the OG is ordered.
    """

    def __init__(context, request, view):
        """adapts context and request and view"""

    def get_title():
        """OG Title"""

    def get_type():
        """OG type"""

    def get_url():
        """OG url"""

    def get_image_url():
        """OG image"""

    def get_site_name():
        """OG site name"""

    def collect_og_data():
        """Returns a ordered dict with all og:key, value"""



from zope.interface import Interface


class IStaging(Interface):
    """The staging adapter provides functionality for implementing and controlling staging.

    It adapts a future baseline, a baseline or a working copy.
    """

    def __init__(context, request):
        pass

    def create_working_copy(target_container):
        """Create a copy of the currently adapted object in the target_container folder.
        Return the working copy object.
        """


class IBaseline(Interface):
    """Marker interface for baseline objects.
    A baseline object is an object of which a working copy exists.
    """


class IWorkingCopy(Interface):
    """Marker interface for working copy objects.
    """



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.interfaces import ISimplelayoutDefaultSettings
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.testbrowser import browsing
from ftw.testbrowser.pages.dexterity import erroneous_fields
from plone import api
from unittest import TestCase
import transaction


class TestImageLimitValidation(TestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def set_config(self, config=[]):
        api.portal.set_registry_record(
            'image_limits', config, ISimplelayoutDefaultSettings)

        transaction.commit()

    @browsing
    def test_raise_invalid_if_hard_limit_is_not_satisfied_for_width(self, browser):
        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock').within(page).with_dummy_image())

        self.set_config({
            block.portal_type: [
                u'hard: width={}'.format(block.image._width + 100)
            ]}
        )

        browser.login().visit(block, view="edit")
        browser.find_button_by_label('Save').click()

        self.assertEqual(
            [["The image doesn't fit the required dimensions of width: 2020px (current: 1920px)"]],
            erroneous_fields().values())

    @browsing
    def test_raise_invalid_if_hard_limit_is_not_satisfied_for_height(self, browser):
        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock').within(page).with_dummy_image())

        self.set_config({
            block.portal_type: [
                u'hard: height={}'.format(block.image._height + 100)
            ]}
        )

        browser.login().visit(block, view="edit")
        browser.find_button_by_label('Save').click()

        self.assertEqual(
            [["The image doesn't fit the required dimensions of height: 1180px (current: 1080px)"]],
            erroneous_fields().values())

    @browsing
    def test_raise_invalid_if_hard_limit_is_not_satisfied_for_width_and_height(self, browser):
        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock').within(page).with_dummy_image())

        self.set_config({
            block.portal_type: [
                u'hard: width={}, height={}'.format(
                    block.image._width + 100,
                    block.image._height + 100)
            ]}
        )

        browser.login().visit(block, view="edit")
        browser.find_button_by_label('Save').click()

        self.assertEqual(
            [["The image doesn't fit the required dimensions of "
              "width: 2020px (current: 1920px) and height: 1180px (current: 1080px)"]],
            erroneous_fields().values())



from AccessControl import getSecurityManager
from Acquisition import aq_inner
from Acquisition import aq_parent
from ftw.simplelayout.interfaces import ISimplelayoutBlock
from ftw.trash.interfaces import IIsRestoreAllowedAdapter
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


@implementer(IIsRestoreAllowedAdapter)
@adapter(ISimplelayoutBlock, Interface)
def is_restoring_bock_allowed(context, request):
    parent = aq_parent(aq_inner(context))
    return bool(getSecurityManager().checkPermission('Modify portal content', parent))



from ftw.simplelayout.interfaces import IContentPageShowTitle
from ftw.simplelayout.interfaces import IPageConfiguration
from ftw.simplelayout.interfaces import ISimplelayoutView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zExceptions import BadRequest
from zope.interface import implements
from zope.publisher.browser import BrowserView
import json
import logging

LOG = logging.getLogger('ftw.simplelayout')


class SimplelayoutView(BrowserView):
    implements(ISimplelayoutView)

    template = ViewPageTemplateFile('templates/simplelayout.pt')

    def __call__(self):
        return self.template()

    def save_state(self):
        data = self.request.form.get('data')
        if data:
            json_conf = json.loads(data)
            page_conf = IPageConfiguration(self.context)
            page_conf.store(json_conf)
        else:
            raise BadRequest('No data given.')

        self.request.response.setHeader("Content-type", "application/json")
        return ''

    def update_simplelayout_settings(self, settings):
        pass

    def show_title(self):
        try:
            adapted_obj = IContentPageShowTitle(self.context)
            return adapted_obj.show_title
        except TypeError:
            return True



from collective import dexteritytextindexer
from ftw.builder import registry
from ftw.builder.dexterity import DexterityBuilder
from ftw.simplelayout.browser.blocks.base import BaseBlock
from ftw.simplelayout.interfaces import ISimplelayout
from ftw.simplelayout.interfaces import ISimplelayoutBlock
from plone.app.textfield import RichText
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.content import Container
from plone.dexterity.content import Item
from plone.dexterity.fti import DexterityFTI
from plone.supermodel import model
from zope import schema
from zope.component import provideAdapter
from zope.interface import alsoProvides
from zope.interface import implements
from zope.interface import Interface
from zope.publisher.interfaces.browser import IBrowserView


# Schemas

class ISampleDXBlockSchema(model.Schema):
    title = schema.TextLine(
        title=u'Title',
        required=False)
    dexteritytextindexer.searchable('text')
    text = RichText(
        title=u'Text',
        required=False,
        allowed_mime_types=('text/html',))

alsoProvides(ISampleDXBlockSchema, IFormFieldProvider)


class ISampleSimplelayoutContainerSchema(model.Schema):
    pass


# Content types

class ISampleDXBlock(Interface):
    pass


class ISampleDXFolderishBlock(Interface):
    pass


class SampleBlock(Item):
    # In Plone 5 the ISimplelayoutBlock marker behavior does not work in tests.
    implements(ISampleDXBlock, ISimplelayoutBlock)


class SampleFolderishBlock(Container):
    implements(ISampleDXFolderishBlock, ISimplelayoutBlock)


class ISampleSimplelayoutContainer(Interface):
    pass


class SampleContainer(Container):
    # In Plone 5 the ISimplelayout marker behavior does not work in tests.
    implements(ISampleSimplelayoutContainer, ISimplelayout)


# Builders


class SampleContainerBuilder(DexterityBuilder):
    portal_type = 'SampleContainer'


class SampleBlockBuilder(DexterityBuilder):
    portal_type = 'SampleBlock'


class SampleFolderishBlockBuilder(DexterityBuilder):
    portal_type = 'SampleFolderishBlock'


registry.builder_registry.register('sample block', SampleBlockBuilder)
registry.builder_registry.register('sample container', SampleContainerBuilder)
registry.builder_registry.register('sample folderish block',
                                   SampleFolderishBlockBuilder)


# Setup

def setup_ftis(portal):

    types_tool = portal.portal_types

    # Simplelayout Container
    fti = DexterityFTI('SampleContainer')
    fti.schema = 'ftw.simplelayout.tests.sample_types.ISampleSimplelayoutContainerSchema'
    fti.klass = 'ftw.simplelayout.tests.sample_types.SampleContainer'
    fti.behaviors = (
        'ftw.simplelayout.interfaces.ISimplelayout',
        'plone.app.dexterity.behaviors.metadata.IBasic',
        'plone.app.content.interfaces.INameFromTitle',
        'collective.dexteritytextindexer.behavior.IDexterityTextIndexer')
    fti.default_view = '@@simplelayout-view'
    types_tool._setObject('SampleContainer', fti)

    # Simplelayout Block
    fti = DexterityFTI('SampleBlock')
    fti.schema = 'ftw.simplelayout.tests.sample_types.ISampleDXBlockSchema'
    fti.klass = 'ftw.simplelayout.tests.sample_types.SampleBlock'
    fti.default_view = 'block_view'
    fti.behaviors = (
        'ftw.simplelayout.interfaces.ISimplelayoutBlock',
        'plone.app.lockingbehavior.behaviors.ILocking',
        'plone.app.content.interfaces.INameFromTitle',)

    types_tool._setObject('SampleBlock', fti)

    # Simplelayout folderish Block
    fti = DexterityFTI('SampleFolderishBlock')
    fti.schema = 'ftw.simplelayout.tests.sample_types.ISampleDXBlockSchema'
    fti.klass = 'ftw.simplelayout.tests.sample_types.SampleFolderishBlock'
    fti.default_view = 'block_view'
    fti.global_allow = False
    fti.behaviors = (
        'ftw.simplelayout.interfaces.ISimplelayoutBlock',
        'plone.app.lockingbehavior.behaviors.ILocking',
        'plone.app.content.interfaces.INameFromTitle',)

    types_tool._setObject('SampleFolderishBlock', fti)
    folderishblock_fti = types_tool.get('SampleFolderishBlock')
    folderishblock_fti.allowed_content_types = ('SampleContainer', )

    contentpage_fti = types_tool.get('SampleContainer')
    contentpage_fti.allowed_content_types = ('SampleBlock', )


def setup_views():

    class SampleBlockView(BaseBlock):

        def __call__(self):
            return 'OK'

    provideAdapter(SampleBlockView,
                   adapts=(ISampleDXBlock, Interface),
                   provides=IBrowserView,
                   name='block_view')

    provideAdapter(SampleBlockView,
                   adapts=(ISampleDXFolderishBlock, Interface),
                   provides=IBrowserView,
                   name='block_view')

    class SampleBlockViewDifferent(BaseBlock):

        def __call__(self):
            return 'OK - different view'

    provideAdapter(SampleBlockViewDifferent,
                   adapts=(ISampleDXBlock, Interface),
                   provides=IBrowserView,
                   name='block_view_different')

    class SampleBlockViewBroken(BaseBlock):

        def __call__(self):
            raise

    provideAdapter(SampleBlockViewBroken,
                   adapts=(ISampleDXBlock, Interface),
                   provides=IBrowserView,
                   name='block_view_broken')



from ftw.upgrade import UpgradeStep


class MakeVideoBlockAvailableForAnonymous(UpgradeStep):
    """Make video block available for anonymous.
    """

    def __call__(self):
        self.install_upgrade_profile()



from collective.geo.mapwidget.browser.widget import MapLayers
from collective.z3cform.mapwidget.widget import FormMapWidget
from collective.z3cform.mapwidget.widget import IFormMapWidget
from collective.z3cform.mapwidget.widget import MapDisplayWidget
from ftw.simplelayout.mapblock.browser.mapblock import BlockMapWidget
from z3c.form.interfaces import DISPLAY_MODE
from z3c.form.interfaces import IFieldWidget
from z3c.form.interfaces import IFormLayer
from z3c.form.widget import FieldWidget
from zope.component import adapter
from zope.interface import implementer
from zope.interface import implementsOnly
from zope.schema.interfaces import IField


class IBlockFormMapWidget(IFormMapWidget):
    """Marker interface for z3c form block map widget"""
    pass


class BlockFormMapWidget(FormMapWidget):
    implementsOnly(IBlockFormMapWidget)

    @property
    def cgmap(self):
        if self.mode == DISPLAY_MODE:
            return MapDisplayWidget(self, self.request, self.context)
        return BlockMapWidget(self, self.request, self.context)


@adapter(IField, IFormLayer)
@implementer(IFieldWidget)
def BlockMapFieldWidget(field, request):
    """IFieldWidget factory for FormMapWidget."""
    return FieldWidget(field, BlockFormMapWidget(request))


class FixedMapLayers(MapLayers):

    @property
    def js(self):
        layers = self.layers()
        return """
$(window).bind('mapload', function (evt, widget) {
    // INFO: No longer load more layers if they're already there.
    if (widget.map.layers.length !== 0) {
        return;
    }

    widget.addLayers([
        %(layers)s
    ], '%(mapid)s');
});

""" % {
            'layers': ",\n".join([l.jsfactory for l in layers]),
            'mapid': self.widget.mapid
        }



from plone.registry.interfaces import IRegistry
from zope.component import getUtility
import pkg_resources


IS_PLONE_5 = pkg_resources.get_distribution('Products.CMFPlone').version >= '5'


def installed(site):
    if IS_PLONE_5:
        add_plone_5_registry(site)


def uninstalled(site):
    if IS_PLONE_5:
        remove_plone_5_registry(site)


def add_plone_5_registry(site):
    registry = getUtility(IRegistry)
    types_not_searched = list(registry['plone.types_not_searched'])
    types_not_searched.append('ftw.simplelayout.AliasBlock')
    registry['plone.types_not_searched'] = tuple(types_not_searched)


def remove_plone_5_registry(site):
    registry = getUtility(IRegistry)
    types_not_searched = list(registry['plone.types_not_searched'])
    types_not_searched.remove('ftw.simplelayout.AliasBlock')
    registry['plone.types_not_searched'] = tuple(types_not_searched)



from functools import wraps
from zExceptions import BadRequest
from zExceptions import Forbidden
from zExceptions import InternalError
from zExceptions import MethodNotAllowed
from zExceptions import NotFound
from zExceptions import Unauthorized
from zope.component.hooks import getSite
import json
import sys
import transaction


class JSONRequestError(Exception):

    def __init__(self, error, status_code=500):
        self.error = error
        self.status_code = status_code


def json_response(request, data=None, **kwargs):
    request.response.setHeader('Content-Type', 'application/json')
    request.response.setHeader('X-Theme-Disabled', 'True')
    return json.dumps(data or kwargs)


def json_error_responses(func):
    def exception_response(status_code, error):
        request = getSite().REQUEST
        getSite().error_log.raising(sys.exc_info())
        transaction.doom()
        request.response.setStatus(status_code)
        return json_response(request, error=error, proceed=False)

    @wraps(func)
    def wrapper(self, REQUEST):
        try:
            return func(self, REQUEST)
        except BadRequest, exc:
            return exception_response(400, str(exc))
        except Unauthorized, exc:
            return exception_response(401, str(exc))
        except Forbidden, exc:
            return exception_response(403, str(exc))
        except NotFound, exc:
            return exception_response(404, str(exc))
        except MethodNotAllowed, exc:
            return exception_response(405, str(exc))
        except (InternalError, Exception), exc:
            return exception_response(500, repr(exc))

    return wrapper



from ftw.upgrade import UpgradeStep


class AddImageCroppingBehaviorToTextBlock(UpgradeStep):
    """Add image cropping behavior to text block.
    """

    def __call__(self):
        self.install_upgrade_profile()



from AccessControl.requestmethod import postonly
from ftw.simplelayout import _
from ftw.simplelayout.browser.ajax.utils import json_error_responses
from ftw.simplelayout.browser.ajax.utils import json_response
from ftw.simplelayout.utils import IS_PLONE_5
from plone import api
from plone.dexterity.interfaces import IDexterityFTI
from plone.dexterity.utils import iterSchemataForType
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import safe_unicode
from Products.Five.browser import BrowserView
from zExceptions import BadRequest
from zope.component import getUtility
from zope.i18n import translate
import os.path

if IS_PLONE_5:
    from plone.namedfile.utils import getImageInfo
else:
    # Plone 4
    from plone.namedfile.file import getImageInfo


class DropzoneUploadBase(BrowserView):

    @json_error_responses
    @postonly
    def __call__(self, REQUEST):
        self.validate_payload()
        obj = self.create(self.request.get('file'))
        return self.created_response(obj)

    def validate_payload(self):
        if 'file' not in self.request.form:
            raise BadRequest('Missing field "file".')

    def created_response(self, obj):
        self.request.response.setStatus(201)
        return json_response(self.request,
                             content='Created',
                             proceed=True,
                             url=obj.absolute_url())

    def is_dexterity_fti(self, portal_type):
        portal_types = getToolByName(self.context, 'portal_types')
        return IDexterityFTI.providedBy(portal_types.get(portal_type))

    def create_obj(self, portal_type, file_field_name, file_):
        filename = safe_unicode(os.path.basename(file_.filename))
        kwargs = {'container': self.context,
                  'type': portal_type,
                  'title': self.request.get('title', filename),
                  'description': self.request.get('description'),
                  'safe_id': True}

        if self.is_dexterity_fti(portal_type):
            field = self.get_field_of_type(portal_type, file_field_name)
            kwargs[file_field_name] = field._type(file_, filename=filename)
            return api.content.create(**kwargs)

        else:
            obj = api.content.create(**kwargs)
            obj.Schema().getField(file_field_name).set(obj, file_)
            obj.reindexObject(idxs=['SearchableText'])
            return obj

    def get_field_of_type(self, portal_type, file_field_name):
        for schema in iterSchemataForType(portal_type):
            if file_field_name in schema:
                return schema[file_field_name]


class FileListingUpload(DropzoneUploadBase):

    def create(self, file_):
        return self.create_obj(self.get_upload_portal_type(), 'file', file_)

    def get_upload_portal_type(self):
        context_fti = getUtility(IDexterityFTI, name=self.context.portal_type)

        if context_fti.allowed_content_types:
            return context_fti.allowed_content_types[0]
        else:
            return 'File'


class GalleryUpload(DropzoneUploadBase):

    def create(self, file_):
        content_type, width, height = getImageInfo(file_.read())
        file_.seek(0)
        if not content_type:
            error = translate(_('Only images can be added to the gallery.'), context=self.request)
            raise BadRequest(error)

        return self.create_obj('Image', 'image', file_)



from ftw.simplelayout.utils import get_block_types
from ftw.simplelayout.utils import normalize_portal_type
from ftw.theming.interfaces import ISCSSResourceFactory
from ftw.theming.resource import DynamicSCSSResource
from zope.interface import provider
import hashlib


TEMPLATE = """
#plone-contentmenu-factories {{
  {0} {{
    display: none !important;
  }}
}}
"""


@provider(ISCSSResourceFactory)
def hide_blocks_in_factory_menu(context, request):

    selectors = []

    for block_fti in get_block_types():
        selectors.append(
            u'.contenttype-{0}'.format(
                normalize_portal_type(block_fti.getId())))

    cachekey = hashlib.md5(''.join(selectors)).hexdigest()

    if len(selectors):
        source = TEMPLATE.format(', '.join(selectors))
    else:
        source = ''

    return DynamicSCSSResource('simplelayout_hide_blocks.scss',
                               slot='addon',
                               source=source,
                               cachekey=cachekey)



from ftw.upgrade import UpgradeStep


class InstallIframefix(UpgradeStep):
    """Install ftw.iframefix.
    """

    def __call__(self):
        self.setup_install_profile('profile-ftw.iframefix:default')
        self.install_upgrade_profile()



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.interfaces import IPageConfiguration
from ftw.simplelayout.interfaces import ISimplelayoutDefaultSettings
from ftw.simplelayout.opengraph.interfaces import IOpenGraphDataProvider
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.testbrowser import browser
from ftw.testbrowser import browsing
from plone import api
from plone.uuid.interfaces import IUUID
from plone.registry.interfaces import IRegistry
from unittest import TestCase
from zope.component import getMultiAdapter
from zope.component import getUtility
import transaction


class TestOpenGraph(TestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def setUp(self):
        super(TestOpenGraph, self).setUp()

        self.portal = self.layer['portal']

    def assertOg(self, key, value):
        ogvalue = browser.css(
            'meta[property="{0}"]'.format(key)).first.attrib['content']

        self.assertEquals(value, ogvalue)

    @browsing
    def test_og_on_plone_root(self, browser):
        browser.exception_bubbling = True
        api.portal.get().manage_changeProperties(
            {'Title': '"<>& Seite title'})
        browser.login().visit()

        self.assertOg('og:title', api.portal.get().Title())
        self.assertOg('og:url', self.portal.absolute_url())
        self.assertOg('og:type', u'website')
        self.assertOg('og:image', self.portal.absolute_url() + '/logo.jpg')

    @browsing
    def test_no_og_site_name_on_plone_root(self, browser):
        browser.login().visit()

        self.assertFalse(len(browser.css('meta[property="og:site_name"]')),
                         'Expect no site_name og property on plone root.')

    def test_disable_og_on_plone_root(self):

        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISimplelayoutDefaultSettings)
        settings.opengraph_plone_root = False

        site_root_og = getMultiAdapter(
            (self.portal, self.portal.REQUEST, None),
            IOpenGraphDataProvider, name='opengraph')

        self.assertEquals([], site_root_og().items())

    @browsing
    def test_og_on_simplelayout_page(self, browser):
        page = create(Builder('sl content page')
                      .titled(u'\xfc Title "<>&')
                      .having(description=u"Description '<>&"))
        browser.login().visit(page)

        self.assertOg('og:title', page.Title().decode('utf-8'))
        self.assertOg('og:description', page.Description().decode('utf-8'))
        self.assertOg('og:url', page.absolute_url())
        self.assertOg('og:type', u'website')
        self.assertOg('og:image', self.portal.absolute_url() + '/logo.jpg')

    @browsing
    def test_og_image_is_leadimage(self, browser):
        page = create(Builder('sl content page').titled(u'\xfc Title'))
        block = create(Builder('sl textblock').with_dummy_image().within(page))
        self.save_state(page, block)

        # Call page once to cache the image scale
        browser.login().visit(page)

        tag = page.restrictedTraverse('@@leadimage')()
        browser.parse(tag)
        src = browser.css('img').first.attrib['src']

        browser.login().visit(page)
        self.assertOg('og:image', src)

    @browsing
    def test_change_og_type(self, browser):
        self.set_og_type(u'food')
        browser.login().visit()
        self.assertOg('og:type', u'food')

    def set_og_type(self, typename):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISimplelayoutDefaultSettings)
        settings.opengraph_global_type = typename
        transaction.commit()

    def save_state(self, page, block):
        self.page_state = {
            "default": [
                {
                    "cols": [
                        {
                            "blocks": [
                                {
                                    "uid": IUUID(block)
                                }
                            ]
                        }
                    ]
                }
            ]
        }
        page_config = IPageConfiguration(page)
        page_config.store(self.page_state)
        transaction.commit()

    @browsing
    def test_opengraph_ftwlogo_integration(self, browser):
        self.portal.portal_setup.runAllImportStepsFromProfile('profile-ftw.logo:default')
        transaction.commit()
        browser.login().visit()

        logo_url_without_cache = '{}/@@logo/logo/get_logo'.format(self.portal.absolute_url())

        og_image = browser.css('meta[property="og:image"]').first.attrib['content']
        browser.open(og_image)
        self.assertTrue(og_image.startswith(logo_url_without_cache))



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.images.interfaces import IImageLimits
from ftw.simplelayout.images.limits.limits import Limits
from ftw.simplelayout.interfaces import ISimplelayoutDefaultSettings
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from plone import api
from plone.namedfile.field import NamedBlobImage
from plone.supermodel import model
from zope.interface import Interface
import transaction
import json


FOO_IMAGE_LIMIT_IDENTIFIER = 'foo'
BAR_IMAGE_LIMIT_IDENTIFIER = 'bar'


class IFooSchema(model.Schema):
    """
    """

    image = NamedBlobImage(
        title=u'image',
        required=False)


class IBarSchema(Interface):
    """
    """

    image = NamedBlobImage(
        title=u'image',
        required=False)


class TestLimits(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def limits(self, config):
        limits = Limits()
        limits.limit_configuration = config

        return limits

    def test_do_not_fail_if_no_configuration_is_set(self):
        api.portal.set_registry_record(
            name='image_limits',
            value={},
            interface=ISimplelayoutDefaultSettings)

        transaction.commit()
        limits = self.limits({})

        self.assertEqual({}, limits.limit_configuration)

    def test_validate_limit_returns_true_if_no_field_is_configured(self):
        limits = self.limits({})
        self.assertTrue(
            limits.validate('soft', FOO_IMAGE_LIMIT_IDENTIFIER, width=10))

    def test_validate_limit_returns_true_if_no_limit_type_is_configured(self):
        limits = self.limits({
            FOO_IMAGE_LIMIT_IDENTIFIER: {}
        })

        self.assertTrue(limits.validate('soft', FOO_IMAGE_LIMIT_IDENTIFIER, width=10))

    def test_validate_limit_returns_true_if_no_width_and_height_is_configured(self):
        limits = self.limits({
            FOO_IMAGE_LIMIT_IDENTIFIER: {
                'soft': {}
            }
        })

        self.assertTrue(limits.validate('soft', FOO_IMAGE_LIMIT_IDENTIFIER, width=10))

    def test_validate_limit_returns_true_if_no_width_and_height_is_given(self):
        limits = self.limits({
            FOO_IMAGE_LIMIT_IDENTIFIER: {
                'soft': {
                    'width': 100,
                    'height': 100
                }
            }
        })

        self.assertTrue(limits.validate('soft', FOO_IMAGE_LIMIT_IDENTIFIER))

    def test_validate_limits(self):
        limits = self.limits({
            FOO_IMAGE_LIMIT_IDENTIFIER: {
                'soft': {
                    'width': 100,
                    'height': 100
                },
                'hard': {
                    'width': 50,
                    'height': 50,
                }
            },
            BAR_IMAGE_LIMIT_IDENTIFIER: {
                'soft': {
                    'width': 500,
                    'height': 500
                },
                'hard': {
                    'width': 250,
                    'height': 250,
                }
            }
        })

        # Soft limit width
        self.assertTrue(limits.validate('soft', FOO_IMAGE_LIMIT_IDENTIFIER, width=200))
        self.assertFalse(limits.validate('soft', FOO_IMAGE_LIMIT_IDENTIFIER, width=70))

        # Hard limit width
        self.assertTrue(limits.validate('hard', FOO_IMAGE_LIMIT_IDENTIFIER, width=70))
        self.assertFalse(limits.validate('hard', FOO_IMAGE_LIMIT_IDENTIFIER, width=30))

        # Soft limit height
        self.assertTrue(limits.validate('soft', BAR_IMAGE_LIMIT_IDENTIFIER, height=600))
        self.assertFalse(limits.validate('soft', BAR_IMAGE_LIMIT_IDENTIFIER, height=400))

        # Hard limit height
        self.assertTrue(limits.validate('hard', BAR_IMAGE_LIMIT_IDENTIFIER, height=300))
        self.assertFalse(limits.validate('hard', BAR_IMAGE_LIMIT_IDENTIFIER, height=200))

        # Soft limit width and height
        self.assertTrue(limits.validate('soft', FOO_IMAGE_LIMIT_IDENTIFIER,
                                        width=200, height=200))
        self.assertFalse(limits.validate('soft', FOO_IMAGE_LIMIT_IDENTIFIER,
                                         width=70, height=70))
        self.assertFalse(limits.validate('soft', FOO_IMAGE_LIMIT_IDENTIFIER,
                                         width=200, height=70))
        self.assertFalse(limits.validate('soft', FOO_IMAGE_LIMIT_IDENTIFIER,
                                         width=70, height=200))

        # Hard limit width and height
        self.assertTrue(limits.validate('soft', BAR_IMAGE_LIMIT_IDENTIFIER,
                                        width=600, height=600))
        self.assertFalse(limits.validate('soft', BAR_IMAGE_LIMIT_IDENTIFIER,
                                         width=70, height=70))
        self.assertFalse(limits.validate('soft', BAR_IMAGE_LIMIT_IDENTIFIER,
                                         width=600, height=70))
        self.assertFalse(limits.validate('soft', BAR_IMAGE_LIMIT_IDENTIFIER,
                                         width=70, height=600))

    def test_get_limits_for_returns_limits(self):
        limits = self.limits({
            FOO_IMAGE_LIMIT_IDENTIFIER: {
                'soft': {
                    'width': 100,
                    'height': 100
                },
                'hard': {
                    'width': 50,
                    'height': 75,
                }
            }
        })

        self.assertEqual(
            {'width': 0, 'height': 0},
            limits.get_limits_for('hard', BAR_IMAGE_LIMIT_IDENTIFIER))

        self.assertDictEqual(
            {'width': 50, 'height': 75},
            limits.get_limits_for('hard', FOO_IMAGE_LIMIT_IDENTIFIER))

    def test_get_all_limits_for_returns_all_registered_limits(self):
        limits = self.limits({
            FOO_IMAGE_LIMIT_IDENTIFIER: {
                'soft': {
                    'width': 100,
                },
                'hard': {
                    'width': 50,
                    'height': 75,
                }
            }
        })

        self.assertDictEqual({
            'hard': {'width': 50, 'height': 75},
            'soft': {'width': 100, 'height': 0}
            },
            limits.get_all_limits_for(FOO_IMAGE_LIMIT_IDENTIFIER)
        )


class TestImageLimits(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def image_limits(self, config, context=None):
        image_limits = IImageLimits(context)
        image_limits.limits.limit_configuration = config

        return image_limits

    def test_validate_limits(self):
        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock').within(page).with_dummy_image())

        image_limits = self.image_limits({
            block.portal_type: {
                'soft': {
                    'width': block.image._width + 100,
                },
                'hard': {
                    'width': block.image._width - 100,
                }
            },
        }, block)

        # Soft limit width
        self.assertFalse(image_limits.validate('soft'))

        # Hard limit width
        self.assertTrue(image_limits.validate('hard'))

    def test_get_limits_for_returns_limits(self):
        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock').within(page).with_dummy_image())

        image_limits = self.image_limits({
            block.portal_type: {
                'soft': {
                    'width': 100,
                }
            }
        }, block)

        self.assertEqual(
            {'width': 100, 'height': 0},
            image_limits.get_limits_for('soft'))

    def test_get_all_limits_for_returns_all_registered_limits(self):
        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock').within(page).with_dummy_image())

        image_limits = self.image_limits({
            block.portal_type: {
                'soft': {
                    'width': 100,
                }
            }
        }, block)

        self.assertDictEqual({
            'hard': {'width': 0, 'height': 0},
            'soft': {'width': 100, 'height': 0}
            },
            image_limits.get_all_limits()
        )

    def test_has_low_quality_image_returns_true_if_soft_limit_not_satisfied(self):
        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock').within(page).with_dummy_image())
        image_limits = self.image_limits({
            block.portal_type: {
                'soft': {
                    'width': block.image._width + 100,
                },
            }
        }, block)

        self.assertTrue(image_limits.has_low_quality_image())

    def test_has_low_quality_image_returns_false_if_soft_limit_is_satisfied(self):
        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock').within(page).with_dummy_image())

        image_limits = self.image_limits({
            block.portal_type: {
                'soft': {
                    'width': block.image._width < 100,
                },
            }
        }, block)

        self.assertFalse(image_limits.has_low_quality_image())



from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from Products.CMFCore.utils import getToolByName
from zope.component.hooks import getSite
from unittest import TestCase


class TestBlocks(TestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def test_blocks_not_searchable(self):
        """
        This test makes sure the blocks are not
        searchable.
        """
        plone_utils = getToolByName(getSite(), 'plone_utils')
        user_friendly_types = plone_utils.getUserFriendlyTypes()

        interface_name = 'ftw.simplelayout.interfaces.ISimplelayoutBlock'
        block_types = filter(
            lambda x: interface_name in getattr(x, 'behaviors', ()),
            getToolByName(getSite(), 'portal_types').objectValues()
        )
        searchable_blocks = filter(
            lambda x: x.__name__ in user_friendly_types,
            block_types
        )
        self.assertEqual(
            [],
            searchable_blocks,
            'Some blocks are searchable. This should not be the case. '
            'Please make them not searchable.'
        )



from ftw.simplelayout.browser.ajax.utils import json_response
from ftw.simplelayout.utils import IS_PLONE_5
from plone import api
from plone.app.uuid.utils import uuidToObject
from plone.uuid.interfaces import IUUID
from Products.CMFPlone.utils import transaction_note
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zExceptions import BadRequest
import json


if IS_PLONE_5:
    from plone.app.linkintegrity.utils import getIncomingLinks

    def isLinked(obj):
        # Important - Check all incoming references, not just tinymce relations
        for it in getIncomingLinks(obj=obj, intid=None, from_attribute=None):
            return True
        return False
else:
    from Products.CMFPlone.utils import isLinked


class DeleteBlocks(BrowserView):

    confirm_template = ViewPageTemplateFile(
        'templates/block_delete_confirmation.pt')

    def __init__(self, context, request):
        super(DeleteBlocks, self).__init__(context, request)
        self.block = None
        self.link_integrity = None

    def __call__(self):
        payload = self.request.get('data', None)
        if not payload:
            raise BadRequest('No data given')

        # TODO validate payload contains blocks and confirmed flag.
        if not IS_PLONE_5:
            from plone.app.linkintegrity.interfaces import ILinkIntegrityInfo
            self.link_integrity = ILinkIntegrityInfo(self.request)

        data = json.loads(payload)
        self.block = uuidToObject(data['block'])

        if self.request.get('form.submitted', False):

            if self.link_integrity:
                # Always allow deletion of block, regardless of the integrity
                # check.
                self.request.environ[self.link_integrity.marker] = 'all'

            self.context.manage_delObjects([self.block.id])
            transaction_note('Deleted %s' % self.block.absolute_url())
            return json_response(self.request, proceed=True)
        else:
            return json_response(self.request,
                                 content=self.confirm_template(),
                                 proceed=False)

    def get_link_integrity_breaches(self):
        if isLinked(self.block):
            breaches_info = []

            if IS_PLONE_5:
                portal = api.portal.get()
                view = api.content.get_view(
                    'delete_confirmation_info',
                    portal,
                    self.request)
                sources = view.get_breaches([self.block, ])[0]['sources']
                for source in sources:
                    breaches_info.append({'title': source['title'],
                                          'url': source['url']})
            else:
                breaches = self.link_integrity.getIntegrityBreaches()
                sources = breaches.values()
                sources = len(sources) and sources[0] or sources

                for source in sources:
                    breaches_info.append({'title': source.title_or_id(),
                                          'url': source.absolute_url()})

            return breaches_info
        else:
            return None

    def is_locked_for_current_user(self):
        locking_info = self.block.restrictedTraverse('@@plone_lock_info', None)
        if locking_info:
            return locking_info.is_locked_for_current_user()
        else:
            return False

    @property
    def context_state(self):
        return self.block.restrictedTraverse('@@plone_context_state')

    def block_payload(self):
        block = IUUID(self.block)
        return json.dumps({'block': block})



from ftw.builder.testing import BUILDER_LAYER
from ftw.builder.testing import functional_session_factory
from ftw.builder.testing import set_builder_session_factory
from ftw.simplelayout.tests import sample_types
from ftw.simplelayout.utils import IS_PLONE_5
from ftw.testing.layer import ComponentRegistryLayer
from path import Path
from persistent.list import PersistentList
from persistent.mapping import PersistentMapping
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import setRoles, TEST_USER_ID, TEST_USER_NAME, login
from plone.testing import z2
from plone.testing import zca
from unittest import TestCase
from zope.configuration import xmlconfig
import ftw.referencewidget.tests.widgets  # noqa
import ftw.simplelayout.tests.builders  # noqa


class SimplelayoutZCMLLayer(ComponentRegistryLayer):

    """A layer which only sets up the zcml, but does not start a zope
    instance.
    """

    defaultBases = (zca.ZCML_DIRECTIVES,)

    def setUp(self):
        super(SimplelayoutZCMLLayer, self).setUp()

        import ftw.simplelayout.tests
        self.load_zcml_file('tests.zcml', ftw.simplelayout.tests)


SIMPLELAYOUT_ZCML_LAYER = SimplelayoutZCMLLayer()


class FtwSimplelayoutLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE, BUILDER_LAYER)

    def setUpZope(self, app, configurationContext):
        xmlconfig.string(
            '<configure xmlns="http://namespaces.zope.org/zope">'
            '  <include package="z3c.autoinclude" file="meta.zcml" />'
            '  <includePlugins package="plone" />'
            '  <includePluginsOverrides package="plone" />'
            '</configure>',
            context=configurationContext)

        z2.installProduct(app, 'ftw.simplelayout')

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'ftw.simplelayout:lib')
        applyProfile(portal, 'plone.restapi:default')

        setRoles(portal, TEST_USER_ID, ['Manager', 'Site Administrator'])
        login(portal, TEST_USER_NAME)


class FtwSimplelayoutContentLayer(FtwSimplelayoutLayer):
    def setUpPloneSite(self, portal):

        applyProfile(portal, 'ftw.simplelayout.contenttypes:default')
        applyProfile(portal, 'ftw.simplelayout.mapblock:default')
        applyProfile(portal, 'ftw.simplelayout.aliasblock:default')
        applyProfile(portal, 'ftw.file:default')

        if IS_PLONE_5:
            applyProfile(portal, 'plone.app.contenttypes:default')

        setRoles(portal, TEST_USER_ID, ['Manager', 'Site Administrator'])
        login(portal, TEST_USER_NAME)


class SimplelayoutTestCase(TestCase):

    def assert_recursive_persistence(self, structure):

        def is_persistent(item):
            if not isinstance(item, basestring) and not isinstance(item, int):  # noqa
                assert isinstance(item, PersistentMapping) or \
                    isinstance(item, PersistentList), \
                    '{0} needs to be persistent'.format(str(item))

                if hasattr(item, 'values'):
                    item = item.values()
                for subitem in item:
                    is_persistent(subitem)
            else:
                return

        is_persistent(structure)

    def assert_unwrapped_persistence(self, structure):
        def is_unwrapped(item):
            if not isinstance(item, basestring) and not isinstance(item, int):  # noqa
                assert isinstance(item, dict) or \
                    isinstance(item, list), \
                    '{0} needs to be unwrapped'.format(str(item))

                if hasattr(item, 'values'):
                    item = item.values()
                for subitem in item:
                    is_unwrapped(subitem)
            else:
                return

        is_unwrapped(structure)

    def setup_sample_ftis(self, portal):
        sample_types.setup_ftis(portal)

    def setup_block_views(self):
        sample_types.setup_views()

    def asset(self, name):
        return Path(__file__).joinpath('..', 'tests', 'assets', name).abspath()


FTW_SIMPLELAYOUT_FIXTURE = FtwSimplelayoutLayer()
FTW_SIMPLELAYOUT_INTEGRATION_TESTING = IntegrationTesting(
    bases=(FTW_SIMPLELAYOUT_FIXTURE,), name="FtwSimplelayout:Integration")
FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FTW_SIMPLELAYOUT_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name='FtwSimplelayout:Functional')

FTW_SIMPLELAYOUT_CONTENT_FIXTURE = FtwSimplelayoutContentLayer()
FTW_SIMPLELAYOUT_CONTENT_TESTING = FunctionalTesting(
    bases=(FTW_SIMPLELAYOUT_CONTENT_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name='FtwSimplelayoutContent:Functional')



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from ftw.testbrowser import browsing
from plone.dexterity.interfaces import IDexterityFTI
from zope.component import getUtility
import transaction


class TestContentPageShowTitleBehavior(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

        self.setup_sample_ftis(self.portal)
        self.setup_block_views()
        fti = getUtility(IDexterityFTI, name='SampleContainer')
        behaviors = list(fti.behaviors)
        behaviors.append('ftw.simplelayout.interfaces.IContentPageShowTitle')
        fti.behaviors = tuple(behaviors)

        self.page = create(Builder('sample container').titled(u'A page'))

    @browsing
    def test_title_doesnt_show_if_show_title_is_false(self, browser):
        browser.login().visit(self.page, view='edit')

        form = browser.find_form_by_field('Show title')
        form.css('input[type="checkbox"]')[0].checked = False
        form.submit()

        self.assertFalse(
            browser.css('.documentFirstHeading').text)

    @browsing
    def test_title_shows_if_show_title_is_true(self, browser):
        browser.login().visit(self.page)

        self.assertEqual(
            ['A page'],
            browser.css('.documentFirstHeading').text)

    @browsing
    def test_allow_and_disallow_changing_show_title_field(self, browser):

        self.page.manage_permission('ftw.simplelayout: Hide title',
                                    roles=[],
                                    acquire=0)
        transaction.commit()

        browser.login().visit(self.page, view='edit')
        self.assertIsNone(browser.find_form_by_field('Show title'),
                          'Do not expect the "Show title" field.')

        self.page.manage_permission('ftw.simplelayout: Hide title',
                                    roles=['Manager'],
                                    acquire=0)
        transaction.commit()

        browser.visit(self.page, view='edit')
        self.assertTrue(browser.find_form_by_field('Show title'),
                        'Exepct the "Show title" field.')



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from ftw.testbrowser import browsing
import transaction


class TestSimplelayoutAsFrontPage(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

        self.setup_sample_ftis(self.portal)
        self.setup_block_views()

        # Make sample block addable on plone root
        types_tool = self.portal.portal_types
        plonesite_fti = types_tool.get('Plone Site')
        addable_types = list(plonesite_fti.allowed_content_types)

        plonesite_fti.allowed_content_types = tuple(
            addable_types + ['ftw.simplelayout.tests.test_ajax_change_block_view.ISampleDX'])
        transaction.commit()

    @browsing
    def test_simplelayout_on_plone_root(self, browser):
        create(Builder('sample block'))
        browser.login().visit(view='@@simplelayout-view')
        self.assertTrue(browser.css('.sl-block-content'))

    def test_addable_types_on_plone_root(self):

        view = self.portal.restrictedTraverse('@@sl-toolbox-view')

        result = [item.get('contentType')
                  for item in list(view.addable_blocks())]

        result.sort()

        self.assertEquals(
            ['sampleblock'],
            result)



from ftw.simplelayout.interfaces import IPageConfiguration
from ftw.simplelayout.interfaces import ISimplelayout
from ftw.upgrade import UpgradeStep
from operator import methodcaller


class UpdateOrderingOfExistingObjects(UpgradeStep):
    """Update ordering of existing objects.
    """

    def __call__(self):
        self.install_upgrade_profile()
        self.update_ordering()

    def update_ordering(self):
        for page in self.objects(
                {'object_provides': ISimplelayout.__identifier__},
                'Update ordering of simplelayout blocks in pages.'):
            IPageConfiguration(page).update_object_positions()



# pylint: disable=E0211, E0213
# E0211: Method has no argument
# E0213: Method should have "self" as first argument

from ftw.simplelayout import _
from plone.autoform import directives
from plone.autoform.interfaces import IFormFieldProvider
from plone.registry import field
from plone.supermodel import model
from zope import schema
from zope.interface import Interface
from zope.interface import provider
from zope.viewlet.interfaces import IViewletManager


class ISimplelayoutLayer(Interface):
    """Browserlayer for simplelayout"""


class ISimplelayout(Interface):
    """Marker for Simplelayout content pages"""


class ISimplelayoutBlock(Interface):
    """Marker for simplelayout blocks"""


@provider(IFormFieldProvider)
class IContentPageShowTitle(model.Schema):
    """Adds checkbox to hide title of a contentpage"""

    show_title = schema.Bool(
        title=_(u'Show title'),
        default=True,
        required=False
    )

    directives.write_permission(
        show_title='ftw.simplelayout.HideTitle')


class ISimplelayoutView(Interface):
    """@@simplelayout-view view interface.
    """


class ISimplelayoutBlockView(Interface):
    """Marker interface for simplelayout block views.
    """


class IPageConfiguration(Interface):
    """Adapter for storing simplelayout page configuration.
    """

    def store(json_conf):
        """Store the given configuration.
        """

    def load():
        """Load current configuration.
        """


class IBlockConfiguration(Interface):
    """Adapter for storing block configuration.
    """

    def store(json_conf):
        """Store the given configuration.
        """

    def load():
        """Load current configuration.
        """


class IBlockProperties(Interface):
    """Adapter for getting and setting information such as the current
    selected view for the block or the available views.
    """

    def __init__(context, request):
        """Adapts context and request.
        """

    def get_current_view_name():
        """Returns the name of the simplelayout view for displaying the
        context.
        """

    def set_view(name):
        """Sets the current view of the context to ``name``.
        """


class ISimplelayoutDefaultSettings(Interface):
    """Stores simplelayout default settings for the hole site"""

    slconfig = schema.Text(
        title=_(u'Simplelayout default configuration'),
        description=(_(
            u'desc_sl_config_control_panel',
            default=u'Add Simplelayout default'
            u'configuration, Check simplelayout'
            u'docu: https://github.com/4teamwork/ftw.simplelayout#usage')),
        default=u'{}',
        required=False)

    opengraph_plone_root = schema.Bool(
        title=_(u'Enable OpenGraph support on plone root'),
        default=True,
        required=False
    )

    opengraph_global_type = schema.TextLine(
        title=_(u'OpenGraph global type'),
        description=_(u'Check possible values on http://ogp.me'),
        default=u'website',
        required=False
    )

    facebook_app_id = schema.TextLine(
        title=_(u'Facebook App ID'),
        description=_(u'Multiple users are separated by comma'),
        default=u'',
        required=False
    )

    facebook_admins = schema.TextLine(
        title=_(u'Facebook Admins'),
        default=u'',
        required=False
    )

    image_limits = schema.Dict(
        title=_(u'Image limits'),
        key_type=schema.ASCIILine(),
        value_type=schema.List(value_type=schema.TextLine()),
        description=_(
            u'desc_image_limits',
            default=u'An image limit will check the image dimensions and validates it agains the '
            u'limit-types.<br><br>'
            u'Use the following configuration-format:<br>'
            u'key: contenttype<br>'
            u'value: limit_type: dimension=value, dimension=value<br><br>'
            u'example:<br>'
            u'key: ftw.simplelayout.TextBlock<br>'
            u'value: soft: width=400, height=300'
            ),
        default={},
        missing_value={},
        required=False
        )

    image_cropping_aspect_ratios = schema.Dict(
        title=_(u'Image cropping aspect ratios'),
        key_type=schema.ASCIILine(),
        value_type=schema.List(value_type=schema.TextLine()),
        description=_(
            u'desc_image_cropping_options',
            default=u'Define the aspect ratios (https://github.com/fengyuanchen/cropperjs#options) '
            u'available for your contenttypes.<br><br>'
            u'Format:<br>'
            u'key: contenttype<br>'
            u'value: title => value<br><br>'
            u'example:<br>'
            u'key: ftw.simplelayout.TextBlock<br>'
            u'value: 4/3 => 1.33333<br><br>'
            u'Calculation: if you want a ratio of 16:9, you need to define the ratio '
            u'to 1.777777778 (16/9 = 1.777777778). 0 means no ratio restrictions.<br><br>'
            ),
        default={'ftw.simplelayout.TextBlock': [u'4:3 => 1.33333', u'16:9 => 1.777777778']},
        missing_value={},
        required=False
        )


class IBlockModifier(Interface):
    """Block specific modifier"""

    def __init__(context, request):
        """Adapts context and request"""

    def modify(data):
        """Modifications based on data in the request"""


class ISimplelayoutActions(Interface):
    """Serves the simplelayout actions"""

    def __init__(context, request):
        """Adapts context and request.
        """

    def default_actions():
        """Default actions"""

    def specific_actions():
        """Specific actions"""


class ISimplelayoutContainerConfig(Interface):
    """Modify simplelayout settings adapter"""

    def __init__(context, request):
        """Adapts context and request.
        """

    def __call__(settings):
        """Receives a settings dict for modification - no return value."""

    def default_page_layout():
        """Define a set of default layouts. A new simplelayout page
        automatically renders the defined layouts.

        Example:
            return {
                "default": [
                    {"cols": [{"blocks": []}]}
                ]
            }

        Results in one layout with one.
        Return None, will trigger a fallback to on layout with one column.
        """


class ISimplelayoutBlockFormManager(IViewletManager):
    """Viewletmanager used to render custom information into add/edit form of
    blocks.
    """



from ftw.referencewidget.selectable import DefaultSelectable
from ftw.referencewidget.sources import ReferenceObjSourceBinder
from ftw.referencewidget.widget import ReferenceBrowserWidget
from ftw.simplelayout import _
from ftw.simplelayout.aliasblock.contents.interfaces import IAliasBlock
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.content import Item
from plone.directives import form
from plone.directives.form import widget
from plone.uuid.interfaces import IUUID
from z3c.form import validator
from z3c.relationfield.schema import RelationChoice
from zope.interface import Invalid
from zope.interface import alsoProvides
from zope.interface import implements


class AliasBlockSelectable(DefaultSelectable):

    def is_selectable(self):
        if not self.content:
            return False

        selectable = super(AliasBlockSelectable, self).is_selectable()
        is_sl_page = self.content.portal_type == 'ftw.simplelayout.ContentPage'

        # Don't allow sl pages containing another AliasBlock
        if is_sl_page:
            if IUUID(self.content) == IUUID(self.source.context):
                return False
            return not bool(filter(
                lambda item: item.portal_type == 'ftw.simplelayout.AliasBlock',
                self.content.objectValues()
            ))
        return selectable


def get_selectable_blocks():
    return ['ftw.simplelayout.TextBlock',
            'ftw.simplelayout.GalleryBlock',
            'ftw.simplelayout.FileListingBlock',
            'ftw.sliderblock.SliderBlock',
            'ftw.news.NewsListingBlock',
            'ftw.events.EventListingBlock',
            'ftw.iframeblock.IFrameBlock',
            'ftw.addressblock.AddressBlock',
            'ftw.simplelayout.MapBlock',
            'ftw.simplelayout.ContentPage']


class IAliasBlockSchema(form.Schema):
    """AliasBlock for simplelayout
    """

    widget('alias', ReferenceBrowserWidget,
           start='parent',
           )
    alias = RelationChoice(
        title=_(u'label_alias_content',
                default=u'Alias Content'),
        description=_(
            u'label_alias_description',
            default=u'Choose a block to be rendered within this block.'),
        required=True,
        source=ReferenceObjSourceBinder(
            selectable_class=AliasBlockSelectable,
            selectable=get_selectable_blocks(),
            override=True
        )
    )


class ContentPageValidator(validator.SimpleFieldValidator):
    """Don't allow sl pages containing another AliasBlock"""

    def validate(self, value):
        """Validate international phone number on input"""
        if not value:
            raise Invalid(
                _(u'error_text_required_aliasblock',
                  default=u'A input is required'))

        if not AliasBlockSelectable(self.field.source, value)():
            if value.portal_type == 'ftw.simplelayout.ContentPage':
                raise Invalid(
                    _(u'error_text_sl_page_aliasblock',
                      default=u'The selected ContentPage contains a Aliasblock or is the page you are creating the block on and thus cannot be selected'))
            else:
                raise Invalid(
                    _(u'error_text_alias_aliasblock',
                      default=u'The selected content cannot be selected'))


validator.WidgetValidatorDiscriminators(
    ContentPageValidator,
    field=IAliasBlockSchema['alias']
)

alsoProvides(IAliasBlockSchema, IFormFieldProvider)


class AliasBlock(Item):
    implements(IAliasBlock)

    def Title(self):
        if not self.alias or self.alias.isBroken():
            return 'AliasBlock: "No target"'
        return 'AliasBlock: "{}"'.format(self.alias.to_object.Title())



from Products.CMFPlone.interfaces import IPatternsSettings
from Products.CMFPlone.patterns.settings import PatternSettingsAdapter
from zope.interface import implementer
import json


@implementer(IPatternsSettings)
class SimplelayoutPatternSettingsAdapter(PatternSettingsAdapter):
    def tinymce(self):
        config = super(SimplelayoutPatternSettingsAdapter, self).tinymce()
        data_config = json.loads(config['data-pat-tinymce'])

        # Original 'prependToUrl': '{0}/resolveuid/'.format(site_path.rstrip('/')),
        data_config['prependToUrl'] = 'resolveuid/'
        return {'data-pat-tinymce': json.dumps(data_config)}



from collections import OrderedDict
from ftw.simplelayout.interfaces import ISimplelayoutDefaultSettings
from ftw.simplelayout.opengraph.interfaces import IOpenGraphDataProvider
from plone import api
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.interfaces import IPloneSiteRoot
from zope.component import adapts
from zope.component import getUtility
from zope.interface import implements
from zope.interface import Interface


class PloneRootOpenGraph(object):
    implements(IOpenGraphDataProvider)
    adapts(IPloneSiteRoot, Interface, Interface)

    def __init__(self, context, request, view):
        """adapts context and request and view"""
        self.context = context
        self.request = request
        self.view = view

    def __call__(self):
        """Returns a dict with all og:key, value"""

        if self._settings.opengraph_plone_root:
            return OrderedDict([
                ('og:title', self.get_title()),
                ('og:type', self.get_type()),
                ('og:url', self.get_url()),
                ('og:image', self.get_image_url()),
                ('og:description', self.get_description()),
                ('fb:app_id', self.get_fb_app_id()),
                ('fb:admins', self.get_fb_admins()),
            ])
        else:
            return OrderedDict([])

    @property
    def _settings(self):
        registry = getUtility(IRegistry)
        return registry.forInterface(
            ISimplelayoutDefaultSettings, check=False)

    def _escape(self, text):
        """More or less taken from https://wiki.python.org/moin/EscapingHtml
        """
        html_escape_table = OrderedDict([
            ("&", "&amp;"),
            ('"', "&quot;"),
            ("'", "&apos;"),
            (">", "&gt;"),
            ("<", "&lt;"),
        ])

        for sign in html_escape_table.items():
            text = text.replace(*sign)
        return text

    def get_title(self):
        """OG Title"""
        return self._escape(api.portal.get().Title().decode('utf-8'))

    def get_type(self):
        """OG type"""

        return self._settings.opengraph_global_type

    def get_url(self):
        """OG url"""
        return api.portal.get().absolute_url().decode('utf-8')

    def get_image_url(self):
        """OG image"""
        return api.portal.get().absolute_url() + '/logo.jpg'

    def get_site_name(self):
        """OG site name"""
        return None

    def get_description(self):
        """OG description"""
        return self._escape(api.portal.get().Description().decode('utf-8'))

    def get_fb_app_id(self):
        """FB app ID"""
        return self._settings.facebook_app_id

    def get_fb_admins(self):
        """FB admins"""
        return self._settings.facebook_admins



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.interfaces import IPageConfiguration
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from ftw.testbrowser import browsing
from plone.uuid.interfaces import IUUID
import re
import transaction


class TestLeadImage(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def setUp(self):
        self.page = create(Builder('sl content page'))
        self.block_without_image = create(Builder('sl textblock')
                                          .within(self.page))

        self.block_with_image = create(Builder('sl textblock')
                                       .within(self.page)
                                       .with_dummy_image())

        self.page_state = {
            "default": [
                {
                    "cols": [
                        {
                            "blocks": [
                                {
                                    "uid": IUUID(self.block_with_image)
                                }
                            ]
                        }
                    ]
                },
                {
                    "cols": [
                        {
                            "blocks": [
                                {
                                    "uid": IUUID(self.block_without_image)
                                }
                            ]
                        }
                    ]
                }
            ]
        }
        self.page_config = IPageConfiguration(self.page)
        self.page_config.store(self.page_state)
        transaction.commit()

    @browsing
    def test_use_first_image_in_page_state(self, browser):
        browser.login().visit(self.page, view='@@leadimage')
        self.assertTrue(
            browser.css('img').first.attrib['src'].startswith(
                self.block_with_image.absolute_url()))

    @browsing
    def test_default_scale_is_preview(self, browser):
        browser.login().visit(self.page, view='@@leadimage')
        self.assertEquals(
            '400',
            browser.css('img').first.attrib['width'])

    @browsing
    def test_render_image_with_other_scale(self, browser):
        browser.login().visit(self.page,
                              view='@@leadimage',
                              data={'scale': 'mini'})
        self.assertEquals(
            '200',
            browser.css('img').first.attrib['width'])

    @browsing
    def test_get_image_from_block_in_any_container(self, browser):
        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock').with_dummy_image().within(page))
        IPageConfiguration(page).store(
            {"portletright": [{"cols": [{"blocks": [
                {"uid": IUUID(block)},
            ]}]}]})
        transaction.commit()

        browser.login().visit(page, view='@@leadimage')
        src_url = browser.css('img').first.attrib['src']
        self.assertRegexpMatches(src_url, r'^{}'.format(
            re.escape(block.absolute_url())))

    @browsing
    def test_support_images_in_gallery_blocks(self, browser):
        page = create(Builder('sl content page'))
        gallery = create(Builder('sl galleryblock').within(page))
        image = create(Builder('image').with_dummy_content().within(gallery))
        page_config = IPageConfiguration(page)
        page_config.store(
            {"default": [{"cols": [{"blocks": [
                {"uid": IUUID(gallery)},
            ]}]}]})
        transaction.commit()

        browser.login().visit(page, view='@@leadimage')
        src_url = browser.css('img').first.attrib['src']
        self.assertRegexpMatches(src_url, r'^{}'.format(
            re.escape(image.absolute_url())))



from ftw.simplelayout.interfaces import IBlockConfiguration
from ftw.simplelayout.interfaces import ISimplelayoutBlockView
from Products.Five.browser import BrowserView
from zope.interface import implements


class BaseBlock(BrowserView):
    implements(ISimplelayoutBlockView)

    blockconfig = None

    def __init__(self, context, request):
        super(BaseBlock, self).__init__(context, request)

        self.blockconfig = IBlockConfiguration(self.context).load()

    def __call__(self):
        return self.template()

    @property
    def block_title(self):
        """
        This property returns the title only if the title should be shown
        and the title has a value.
        """
        title = getattr(self.context, 'title', '')
        show_title = getattr(self.context, 'show_title', True)

        if show_title and title:
            return title
        return ''



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.interfaces import IBlockModifier
from ftw.simplelayout.interfaces import IBlockProperties
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_INTEGRATION_TESTING
from ftw.simplelayout.tests.sample_types import ISampleDXBlock
from ftw.simplelayout.testing import SimplelayoutTestCase
from plone.uuid.interfaces import IUUID
from zExceptions import BadRequest
from zope.component import provideAdapter
from zope.interface import implements
from zope.interface import Interface
import json


class TestBlockReloadView(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.setup_sample_ftis(self.portal)
        self.setup_block_views()
        self.container = create(Builder('sample container'))

    def get_reload_view(self, block, payload={}):
        payload['uid'] = IUUID(block)
        self.container.REQUEST.set('data', json.dumps(payload))
        return self.container.restrictedTraverse(
            '@@sl-ajax-reload-block-view')

    def test_getting_block_with_reload_view(self):
        block = create(Builder('sample block').within(self.container))
        reload_view = self.get_reload_view(block)
        reload_view()

        self.assertEquals(block, reload_view.block)

    def test_getting_block_properties_with_reload_view(self):
        block = create(Builder('sample block').within(self.container))
        reload_view = self.get_reload_view(block)
        reload_view()

        self.assertTrue(IBlockProperties.providedBy(reload_view.properties))

    def test_raise_bad_request_if_no_uid_is_in_payload(self):
        reload_view = self.container.restrictedTraverse(
            '@@sl-ajax-reload-block-view')
        with self.assertRaises(BadRequest):
            reload_view()

    def test_return_value_is_the_block_content(self):
        block = create(Builder('sample block').within(self.container))
        reload_view = self.get_reload_view(block)

        self.assertEquals('OK', reload_view())

    def test_changing_block_view(self):
        block = create(Builder('sample block').within(self.container))
        reload_view = self.get_reload_view(
            block,
            dict(view_name='block_view_different'))

        self.assertEquals('OK - different view', reload_view())

    def test_block_specific_modifier_gets_called(self):

        class SampleBlockModifier(object):
            """Sets the attribute foo to the given value"""

            implements(IBlockModifier)

            def __init__(self, context, request):
                self.context = context
                self.request = request

            def modify(self, data):
                setattr(self.context, 'foo', data.get('foovalue', None))

        provideAdapter(SampleBlockModifier,
                       adapts=(ISampleDXBlock, Interface),
                       provides=IBlockModifier)

        block = create(Builder('sample block').within(self.container))
        reload_view = self.get_reload_view(
            block,
            dict(foovalue='bar'))
        reload_view()

        self.assertEquals('bar', block.foo)



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.interfaces import IPageConfiguration
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from ftw.testing import staticuid
import transaction


class TestPageBuilder(SimplelayoutTestCase):
    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    @staticuid('staticuid')
    def test_page_builder_with_blocks(self):
        page = create(Builder('sl content page')
                      .with_blocks(Builder('sl textblock'),
                                   Builder('sl textblock')))

        # reset transaction to verify that page state is comitted
        transaction.begin()
        self.assertEquals(
            {'default': [{'cols': [{'blocks': [
                {'uid': 'staticuid00000000000000000000002'},
                {'uid': 'staticuid00000000000000000000003'}]}]}]},
            IPageConfiguration(page).load())



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.simplelayout.testing import IS_PLONE_5
from ftw.testbrowser import browsing
from unittest import skipUnless
from unittest import TestCase
import transaction


class TestMapBlock(TestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def setUp(self):
        super(TestMapBlock, self).setUp()
        self.portal = self.layer['portal']
        self.page = create(Builder('sl content page').titled(u'A page'))

    @browsing
    def test_mapblock_title_is_rendered_by_default(self, browser):
        block = create(Builder('sl mapblock')
                       .titled('My mapblock')
                       .within(self.page))

        browser.login().visit(self.page)

        title_css_selector = '.ftw-simplelayout-mapblock h2'

        # The title is rendered by default.
        self.assertEqual('My mapblock',
                         browser.css(title_css_selector).first.text)

        # Disable the rendering of the title.
        browser.visit(block, view='edit')
        browser.fill({'Show title': False}).save()
        self.assertEqual([], browser.css(title_css_selector))

    @browsing
    def test_mapblock_title_not_rendered_when_empty(self, browser):
        """
        Makesure that the title of the block is only rendered
        if there is a title. Otherwise we'll end up with an empty HTML
        tag in the template.
        """
        block = create(Builder('sl mapblock')
                       .titled('My mapblock')
                       .within(self.page))

        browser.login().visit(self.page)

        title_css_selector = '.ftw-simplelayout-mapblock h2'

        # The title is rendered by default.
        self.assertEqual('My mapblock',
                         browser.css(title_css_selector).first.text)

        # Remove the title of the block and make sure the tag is no longer
        # there.
        block.title = ''
        transaction.commit()
        browser.login().visit(self.page)
        self.assertEqual([], browser.css(title_css_selector))



from collections import OrderedDict
from ftw.simplelayout import _
from ftw.simplelayout.browser.actions import DefaultActions
from ftw.simplelayout.contenttypes.contents import interfaces
from ftw.table import helper
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.content import Container
from plone.directives import form
from zope import schema
from zope.component import adapts
from zope.component import queryMultiAdapter
from zope.i18n import translate
from zope.interface import alsoProvides
from zope.interface import directlyProvides
from zope.interface import implements
from zope.interface import Interface
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


def download_link(icon=True, classes=None, attrs=None, icon_only=False):

    def _helper(item, value):
        url = '%s/download' % item.getURL()
        attrs = {}
        attrs['href'] = url
        attrs['title'] = item.Description
        return helper.linked(item, value, show_icon=icon,
                             attrs=attrs, icon_only=icon_only)
    return _helper


class ListingBlockDefaultColumns(object):
    adapts(Interface, Interface)
    implements(interfaces.IListingBlockColumns)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def columns(self):
        columns = (
            {'column': 'getContentType',
             'column_title': _(u'column_type', default=u'Type'),
             'sort_index': 'portal_type',
             'transform': download_link(icon=True, icon_only=True)},

            {'column': 'Title',
             'column_title': _(u'column_title', default=u'Title'),
             'sort_index': 'sortable_title',
             'transform': download_link(icon=False)},

            {'column': 'modified',
             'column_title': _(u'column_modified', default=u'modified'),
             'sort_index': 'modified',
             'transform': helper.readable_date,
             },

            {'column': 'Creator',
             'column_title': _(u'column_creater', default=u'creater'),
             'transform': helper.readable_author,
             },

            {'column': 'getObjSize',
             'column_title': _(u'column_size', default=u'size'),
             },

            {'column': 'id',
             'column_title': _(u'ID', default=u'ID'),
             'sort_index': 'id',
             },
        )
        return columns


def listing_block_columns(context):
    terms = []
    adapter = queryMultiAdapter((context, context.REQUEST),
                                interfaces.IListingBlockColumns)

    for column in adapter.columns():
        terms.append(
            SimpleVocabulary.createTerm(
                column['column'],
                column['column'],
                column['column_title']))

    return SimpleVocabulary(terms)

directlyProvides(listing_block_columns, IContextSourceBinder)


def sort_index_vocabulary(context):
    adapter = queryMultiAdapter((context, context.REQUEST),
                                interfaces.IListingBlockColumns)
    terms = []
    for col in adapter.columns():
        if 'sort_index' in col:
            terms.append(SimpleVocabulary.createTerm(
                col['sort_index'],
                col['sort_index'],
                _(u'label_%s' % col['sort_index'],
                  default=col['sort_index'])))
    terms.append(SimpleVocabulary.createTerm(
        'getObjPositionInParent',
        'getObjPositionInParent',
        _(u'label_position_in_folder',
          default=u'Position in Folder')))

    return SimpleVocabulary(terms)

directlyProvides(sort_index_vocabulary, IContextSourceBinder)


sort_order_vocabulary = SimpleVocabulary([
    SimpleTerm(value='ascending',
               title=_(u'label_ascending', default=u'Ascending')),
    SimpleTerm(value='descending',
               title=_(u'label_descending', default=u'Descending'))
])


class IFileListingBlockSchema(form.Schema):
    """ListingBlock for simplelayout
    """

    title = schema.TextLine(
        title=_(u'label_title', default=u'Title'),
        required=False)

    show_title = schema.Bool(
        title=_(u'label_show_title', default=u'Show title'),
        default=True,
        required=False)

    columns = schema.List(
        title=_(u'label_columns', default=u'Columns'),
        value_type=schema.Choice(source=listing_block_columns),
        required=True,
        default=['getContentType', 'Title', 'modified'])

    sort_on = schema.Choice(
        title=_(u'label_sort_on', default=u'Sort by'),
        required=True,
        default="sortable_title",
        source=sort_index_vocabulary)

    sort_order = schema.Choice(
        title=_(u'label_sort_order', default=u'Sort order'),
        required=True,
        default="ascending",
        vocabulary=sort_order_vocabulary)


alsoProvides(IFileListingBlockSchema, IFormFieldProvider)


class FileListingBlock(Container):
    implements(interfaces.IFileListingBlock)


class ListingBlockActions(DefaultActions):

    def specific_actions(self):
        return OrderedDict([
            ('folderContents', {
                'class': 'sl-icon-folder-contents redirect',
                'title': translate(
                    _(u'label_folder_contents_files',
                      default=u'Go to folder contents for managing files'),
                    context=self.request),
                'href': '/folder_contents'
            }),
        ])



from Acquisition._Acquisition import aq_inner
from ftw.simplelayout import _
from ftw.simplelayout.browser.blocks.base import BaseBlock
from plone import api
from plone.app.imaging.scale import ImageScale
from plone.app.imaging.utils import getAllowedSizes
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import safe_unicode
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.i18n import translate
import os


class GalleryBlockView(BaseBlock):
    """GalleryBlock default view"""

    template = ViewPageTemplateFile('templates/galleryblock.pt')

    def get_images(self):
        imgBrains = self.context.portal_catalog.searchResults(
            portal_type="Image",
            sort_on=self.context.sort_on,
            sort_order=self.context.sort_order,
            path='/'.join(self.context.getPhysicalPath()))
        images = []
        for img in imgBrains:
            images.append(img.getObject())
        return images

    def get_box_boundaries(self):
        return getAllowedSizes().get('simplelayout_galleryblock')

    @property
    def can_add(self):
        context = aq_inner(self.context)
        mtool = getToolByName(context, 'portal_membership')
        permission = mtool.checkPermission(
            'ftw.simplelayout: Add GalleryBlock', context)
        return bool(permission)

    def generate_image_alttext(self, img):
        title = safe_unicode(img.title_or_id())
        return translate(_(u'image_link_alttext',
                           default=u'${title}, enlarged picture.',
                           mapping={'title': title}),
                         context=self.request)

    def get_full_image_scale(self, img):
        scales = img.restrictedTraverse('@@images')
        try:
            scale = scales.scale('image', scale='sl_galleryblock_4k', direction='up')
        except IOError:
            scale = None

        if not scale:
            scale = self._get_fallback_image_scale()

        return scale.absolute_url()

    def get_image_scale_tag(self, img):
        scales = img.restrictedTraverse('@@images')
        try:
            scale = scales.scale('image', scale='simplelayout_galleryblock', direction='down')
        except IOError:
            scale = None

        if not scale:
            scale = self._get_fallback_image_scale()

        return u'<img src="{src}" height="{height}" width="{width}" alt="{alt}"/>'.format(
            src=scale.absolute_url(),
            height=scale.height,
            width=scale.width,
            alt=self.generate_image_alttext(img),
        )

    def _get_fallback_image_scale(self):
        current_folder = os.path.dirname(__file__)

        fallback_image_path = os.path.join(
            os.path.dirname(os.path.dirname(current_folder)),
            'browser',
            'resources',
            'image_unavailable.png'
        )

        with open(fallback_image_path, 'rb') as fallback_image:
            fallback_image_scale = FallbackImageScale(
                id='galleryblock_fallback_image',
                title='',
                data=fallback_image.read(),
                content_type='image/png',
            )

        return fallback_image_scale


class FallbackImageScale(ImageScale):
    def absolute_url(self):
        return api.portal.get().absolute_url() + '/++resource++ftw.simplelayout/image_unavailable.png'



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.contenttypes.contents.filelistingblock import listing_block_columns
from ftw.simplelayout.contenttypes.contents.filelistingblock import ListingBlockDefaultColumns
from ftw.simplelayout.contenttypes.contents.interfaces import IListingBlockColumns
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import factoriesmenu
from plone.app.testing import TEST_USER_ID
from unittest import skip
from unittest import TestCase
from zope.component import queryMultiAdapter
from zope.interface.verify import verifyClass
from zope.schema.vocabulary import SimpleVocabulary
import transaction


def assert_ftw_table_column(column):
    assert 'column' in column, 'Expect column in {0}'.format(
        str(column))
    assert 'column_title' in column, 'Expect column_title in {0}'.format(
        str(column))


class TestListingBlock(TestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def setUp(self):
        super(TestListingBlock, self).setUp()
        self.portal = self.layer['portal']
        self.page = create(Builder('sl content page').titled(u'A page'))

    def test_listingblock_default_columns_adapter(self):
        verifyClass(IListingBlockColumns, ListingBlockDefaultColumns)

        adapter = queryMultiAdapter((self.portal, self.portal.REQUEST),
                                    IListingBlockColumns)

        self.assertIsNotNone(adapter)

        for column in adapter.columns():
            assert_ftw_table_column(column)

    def test_columns_context_source_binder(self):

        vocabulary = listing_block_columns(self.portal)
        self.assertTrue(isinstance(vocabulary, SimpleVocabulary),
                        'Expect a SimpleVocabulary instance.')

    @browsing
    def test_columns_vocabulary_on_listingblock_add_form(self, browser):
        browser.login().visit(self.page)
        factoriesmenu.add('FileListingBlock')

        self.assertEquals(
            ['creater', 'size', 'ID'],
            browser.css('#form-widgets-columns-from option').text)

        self.assertEquals(
            ['Type', 'Title', 'modified'],
            browser.css('#form-widgets-columns-to option').text)

    @skip('How do I use this ordered multi select widget without js?')
    @browsing
    def test_adding_listingblock(self, browser):
        browser.login().visit(self.page)
        factoriesmenu.add('FileListingBlock')
        browser.fill(
            {'Title': 'My listingblock', 'Show title': True}).submit()
        self.assertEquals(self.page.absolute_url(), browser.url)

    @browsing
    def test_listingblock_rendering(self, browser):
        create(Builder('sl listingblock')
               .titled('My listingblock')
               .having(show_title=True)
               .within(self.page))

        browser.login().visit(self.page)

        self.assertTrue(browser.css('.sl-block'))

        self.assertEquals(u'My listingblock',
                          browser.css('.sl-block h2').first.text)

        self.assertEquals(['Type', 'Title', 'modified'],
                          browser.css('.sl-block table th').text)

    @browsing
    def test_listingblock_table_contents(self, browser):
        block = create(Builder('sl listingblock')
                       .titled('My listingblock')
                       .having(show_title=True)
                       .within(self.page))

        file_ = create(Builder('file')
                       .titled('Test file')
                       .having(creators=(TEST_USER_ID.decode('utf-8'), ))
                       .with_dummy_content()
                       .within(block))

        modified = file_.modified().strftime('%d.%m.%Y')
        browser.login().visit(self.page)
        self.assertEquals([['Type', 'Title', 'modified'],
                           ['', 'Test file', modified]],
                          browser.css('.sl-block table').first.lists())

    @browsing
    def test_listingblock_title_not_rendered_when_empty(self, browser):
        """
        This test makes sure that the title of the block is only rendered
        if there is a title. Otherwise we'll end up with an empty HTML
        tag in the template.
        """
        listingblock = create(Builder('sl listingblock')
                              .titled('My listingblock')
                              .having(show_title=True)
                              .within(self.page))

        browser.login().visit(self.page)

        title_css_selector = '.ftw-simplelayout-filelistingblock h2'

        # Make sure the title is here (in its tag).
        self.assertEqual('My listingblock',
                         browser.css(title_css_selector).first.text)

        # Remove the title of the block and make sure the tag is no longer
        # there.
        listingblock.title = ''
        transaction.commit()
        browser.login().visit(self.page)
        self.assertEqual([], browser.css(title_css_selector))

    @browsing
    def test_hidden_listingblock_has_special_class(self, browser):
        """
        This test makes sure that a special class is available on the block
        if the block is hidden. This can be used to visually highlight
        hidden blocks.
        """
        listingblock = create(Builder('sl listingblock')
                              .titled('My listingblock')
                              .having(show_title=True)
                              .having(is_hidden=True)
                              .within(self.page))

        browser.login()

        # The block must have a class "hidden".
        browser.visit(self.page)
        self.assertEqual(
            'sl-block ftw-simplelayout-filelistingblock hidden',
            browser.css('.ftw-simplelayout-filelistingblock').first.attrib['class']
        )

        # Edit the block and make it appear again.
        browser.visit(listingblock, view='edit')
        browser.fill({'Hide the block': False, 'Columns': 'Type'}).submit()

        # The block must no longer have a class "hidden".
        browser.visit(self.page)
        self.assertEqual(
            'sl-block ftw-simplelayout-filelistingblock',
            browser.css('.ftw-simplelayout-filelistingblock').first.attrib['class']
        )

    @browsing
    def test_hidden_listingblock_not_visible_without_edit_permission(self, browser):
        """
        This test makes sure that users without edit permission, e.g. the
        anonymous user, do not see the hidden block.
        """
        listingblock = create(Builder('sl listingblock')
                              .titled('My listingblock')
                              .having(show_title=True)
                              .having(is_hidden=True)
                              .within(self.page))

        # Make sure an anonymous user cannot see the block.
        browser.logout().visit(self.page)
        self.assertEqual(
            [],
            browser.css('.ftw-simplelayout-filelistingblock')
        )

        # Login to make sure the block is visible for admin users.
        browser.login().visit(self.page)
        self.assertEqual(
            ['My listingblock'],
            browser.css('.ftw-simplelayout-filelistingblock h2').text
        )



# pylint: disable=E0211, E0213
# E0211: Method has no argument
# E0213: Method should have "self" as first argument

from zope.interface import Interface


class IMapBlock(Interface):
    """Marker interface for MapBlocks"""



from ftw.simplelayout import _
from ftw.simplelayout.contenttypes.contents.interfaces import IVideoBlock
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.content import Item
from plone.directives import form
from urlparse import urlparse
from zope import schema
from zope.interface import alsoProvides
from zope.interface import implements
from zope.interface import Invalid
from zope.interface import invariant


def is_vimeo_url(url):
    # https://vimeo.com/channels/staffpicks/128510631
    parsed_url = urlparse(url)
    path = parsed_url.path.split('/')
    return parsed_url.netloc == 'vimeo.com' and path[-1].isdigit()


def is_youtube_url(url):
    # https://youtu.be/W42x6-Wf3Cs
    parsed_url = urlparse(url)
    path = parsed_url.path.split('/')
    return parsed_url.netloc == 'youtu.be' and len(path) == 2 and path[-1]


def is_youtube_nocookie_url(url):
    # https://www.youtube-nocookie.com/embed/UUrddqT9i_s
    parsed_url = urlparse(url)
    path = parsed_url.path.split('/')
    return parsed_url.netloc == 'www.youtube-nocookie.com' and len(path) == 3 and path[-1]


class IVideoBlockSchema(form.Schema):
    """VideoBlock for simplelayout
    """

    title = schema.TextLine(
        title=_(u'label_title', default=u'Title'),
        required=False)

    show_title = schema.Bool(
        title=_(u'label_show_title', default=u'Show title'),
        default=False,
        required=False)

    video_url = schema.URI(
        title=_(u'label_video_url', default=u'Youtube, or Vimeo URL'),
        description=_(u'Youtube format: http(s)://youtu.be/VIDEO_ID (add start parameter to let '
                      u'the vide start at a specific position)<br/>'
                      u'Youtube (no-cookie) format: https://www.youtube-nocookie.com/embed/VIDEO_ID<br/>'
                      u'Vimeo format: http(s)://vimeo.com/(channels/groups)/'
                      u'VIDEO_ID'),
        required=True)

    player_height = schema.TextLine(
        title=_(u'label_height', default=u'Fixed hight for Vimeo and Youtube nocookie'),
        required=False)

    @invariant
    def validate_video_url(data):
        if is_youtube_url(data.video_url):
            return
        elif is_vimeo_url(data.video_url):
            return
        elif is_youtube_nocookie_url(data.video_url):
            return
        else:
            raise Invalid(_(u'This is no a valid youtube, or vimeo url.'))


alsoProvides(IVideoBlockSchema, IFormFieldProvider)


class VideoBlock(Item):
    implements(IVideoBlock)



from ftw.simplelayout.interfaces import IBlockProperties
from Persistence import PersistentMapping
from plone.dexterity.utils import safe_utf8
from zope.annotation import IAnnotations
from zope.interface import implements


BLOCK_PROPERTIES_KEY = 'ftw.simplelayout.block_properites'


class MultiViewBlockProperties(object):
    """Block properites adapter for blocks where the user can choose from
    multiple views.
    """

    implements(IBlockProperties)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def get_current_view_name(self):
        return self.get_storage().get('view-name', 'block_view')

    def set_view(self, name):
        if not self.is_view_available(name):
            raise ValueError('"{0}" is not in available views.'.format(name))

        annotations = IAnnotations(self.context)
        if BLOCK_PROPERTIES_KEY not in annotations:
            annotations[BLOCK_PROPERTIES_KEY] = PersistentMapping()

        self.get_storage()['view-name'] = name

    def get_storage(self):
        annotations = IAnnotations(self.context)
        return annotations.get(BLOCK_PROPERTIES_KEY, {})

    def is_view_available(self, name):
        return self.context.restrictedTraverse(safe_utf8(name), None) is not None



from ftw.upgrade import UpgradeStep


class FixSimplelayoutImageScalingDimensions(UpgradeStep):
    """Fix simplelayout image scaling dimensions.
    """

    def __call__(self):
        self.install_upgrade_profile()



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.configuration import columns_in_config
from ftw.simplelayout.configuration import synchronize_page_config_with_blocks
from ftw.simplelayout.interfaces import IPageConfiguration
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase


class TestOrdering(SimplelayoutTestCase):
    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def test_blocks_are_ordered_by_simplelayout_order(self):
        page = create(
            Builder('sl content page').titled(u'Page')
            .with_blocks(Builder('sl textblock').titled(u'Block 1'),
                         Builder('sl textblock').titled(u'Block 2')))

        self.assert_order(page, 'block-1', 'block-2')
        self.reverse_sl_block_order(page)
        self.assert_order(page, 'block-2', 'block-1')

    def test_non_blocks_are_ordered_normally(self):
        page = create(
            Builder('sl content page').titled(u'Page')
            .with_blocks(Builder('sl content page').titled(u'Page 1'),
                         Builder('sl content page').titled(u'Page 2')))

        self.assert_order(page, 'page-1', 'page-2')
        page.getOrdering().moveObjectsToTop(['page-2'])
        self.assert_order(page, 'page-2', 'page-1')

    def test_changing_blocks_moves_them_to_top(self):
        page = create(
            Builder('sl content page').titled(u'Page')
            .with_blocks(Builder('sl textblock').titled(u'Block 1'),
                         Builder('sl content page').titled(u'Page 1'),
                         Builder('sl textblock').titled(u'Block 2'),
                         Builder('sl content page').titled(u'Page 2')))

        self.assert_order(page, 'block-1', 'block-2', 'page-1', 'page-2')
        page.getOrdering().moveObjectsToTop(['page-2'])
        self.assert_order(page, 'page-2', 'block-1', 'block-2', 'page-1')
        self.reverse_sl_block_order(page)
        self.assert_order(page, 'block-2', 'block-1', 'page-2', 'page-1')

    def test_orering_on_site_root(self):
        portal = self.layer['portal']
        create(Builder('sl textblock').titled(u'Block 1').within(portal))
        create(Builder('sl textblock').titled(u'Block 2').within(portal))
        synchronize_page_config_with_blocks(portal)

        self.assertEquals(['block-1', 'block-2'],
                          filter(lambda id_: id_.startswith('block'),
                                 portal.objectIds()))
        self.reverse_sl_block_order(portal)
        self.assertEquals(['block-2', 'block-1'],
                          filter(lambda id_: id_.startswith('block'),
                                 portal.objectIds()))

    def reverse_sl_block_order(self, container):
        config = IPageConfiguration(container).load()
        for column in columns_in_config(config):
            column['blocks'].reverse()

        config = IPageConfiguration(container).store(config)

    def assert_order(self, container, *child_ids):
        self.assertEquals(list(child_ids), container.objectIds())
        self.assert_positions_consistent(container)

    def assert_positions_consistent(self, container):
        """Make sure that the positions are consistently incremented.
        This is verified on order to avoid problems when using multiple
        ordering strategies in combination.
        """
        ordering = container.getOrdering()
        ids = ordering.idsInOrder()
        self.assertEquals(range(len(ids)),
                          map(ordering.getObjectPosition, ids))



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.interfaces import IPageConfiguration
from ftw.simplelayout.portlets.portlet import Assignment
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from ftw.testbrowser import browsing
from plone.app.textfield.value import RichTextValue
from plone.portlets.interfaces import IPortletAssignmentMapping
from plone.portlets.interfaces import IPortletRenderer
from plone.portlets.interfaces import IPortletManager
from plone.uuid.interfaces import IUUID
from zope.component import getMultiAdapter
from zope.component import getUtility
import transaction


class TestSimplelayoutPortlet(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.setup_sample_ftis(self.portal)
        self.setup_block_views()

        self.container = create(Builder('sample container'))
        self.block = create(Builder('sample block')
                            .titled('TextBlock title')
                            .within(self.container)
                            .having(text=RichTextValue('The text'))
                            )

        create(Builder('sample block')
               .titled('TextBlock not in portlet')
               .within(self.container)
               )

    def setup_portlet(self, manager='plone.rightcolumn'):
        manager = getUtility(IPortletManager, name=manager)
        mapping = getMultiAdapter((self.container, manager),
                                  IPortletAssignmentMapping)
        mapping['sl-portlet'] = Assignment()
        transaction.commit()

    def setup_block(self, container='portletright'):
        page_config = IPageConfiguration(self.container)
        page_config.store({container: [
            {
                "cols": [
                    {
                        "blocks": [
                            {
                                "uid": IUUID(self.block)
                            }
                        ]
                    }
                ]
            }]})
        transaction.commit()

    @browsing
    def test_simplelayout_portlet_on_the_right(self, browser):
        self.setup_portlet()
        self.setup_block()
        browser.login().visit(self.container)

        self.assertTrue(browser.css('.portlet.SimplelayoutPortlet'),
                        'There should be a simplelayout portlet.')

        self.assertEquals(
            'OK',
            browser.css('.portlet.SimplelayoutPortlet .sl-block').first.text)

    @browsing
    def test_simplelayout_portlet_on_the_left(self, browser):
        self.setup_portlet(manager='plone.leftcolumn')
        self.setup_block(container='portletleft')
        browser.login().visit(self.container)

        self.assertTrue(browser.css('.portlet.SimplelayoutPortlet'),
                        'There should be a simplelayout portlet.')

        self.assertEquals(
            'OK',
            browser.css('.portlet.SimplelayoutPortlet .sl-block').first.text)

    @browsing
    def test_portlet_is_not_visible_when_empty_if_user_cannot_edit(self, browser):
        self.setup_portlet()
        browser.visit(self.container)

        self.assertFalse(browser.css('.portlet.SimplelayoutPortlet'),
                         'Portlet is empty and should not be visible')

    @browsing
    def test_portlet_is_not_visible_when_empty_and_disable_border_is_true(self, browser):
        self.setup_portlet()
        browser.login().visit(self.container, data={'disable_border': 1})

        self.assertFalse(browser.css('.portlet.SimplelayoutPortlet'),
                         'Portlet is empty and should not be visible')

    def test_simplelayout_portlet_not_available_on_non_simplelayout_types(self):
        """
        This test makes sure that the simplelayout portlet is not available on
        non-simplelayout items.

        This prevents the logs from flooding with error like:

            TypeError:
                'Could not adapt',
                <ATLink at /plone/samplecontainer/link used for /plone/samplecontainer/link>,
                <InterfaceClass ftw.simplelayout.interfaces.IPageConfiguration>
        """
        self.setup_portlet()
        self.setup_block()

        # Add "Link" to addable types of "SampleContainer".
        fti = self.portal.portal_types.get('SampleContainer')
        allowed_content_types = list(fti.allowed_content_types)
        allowed_content_types += ['File']
        fti.allowed_content_types = tuple(set(allowed_content_types))
        transaction.commit()

        # Create a link below the container where the portlet is assigned.
        link = create(Builder('file').within(self.container))

        # Get the portlet renderer on the link.
        request = self.layer['request']
        view = link.restrictedTraverse('@@plone')
        manager = getUtility(IPortletManager, name='plone.rightcolumn', context=link)
        assignment = Assignment()
        renderer = getMultiAdapter(
            (link, request, view, manager, assignment),
            IPortletRenderer
        )

        # Make sure the portlet is not available.
        self.assertFalse(renderer.available)



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from ftw.simplelayout.utils import IS_PLONE_5
from ftw.testbrowser import browser as defaultbrowser
from ftw.testbrowser import browsing
from plone.dexterity.interfaces import IDexterityFTI
from plone.protect import createToken
from zope.component import getUtility


class TestDropZoneUpload(SimplelayoutTestCase):
    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    @browsing
    def test_create_files_in_filelistingblock(self, browser):
        listing = create(Builder('sl listingblock').titled(u'Downloads')
                         .within(create(Builder('sl content page')
                                 .titled(u'Page'))))
        self.assertEquals([], listing.objectIds())

        browser.login()
        self.make_dropzone_upload(listing, self.asset('world.txt').open('rb'))
        self.assertEqual(201, browser.status_code)
        self.assertEqual({u'content': u'Created',
                          u'url': u'http://nohost/plone/page/downloads/world.txt',
                          u'proceed': True}, browser.json)
        self.assertEquals(['world.txt'], listing.objectIds())

        doc, = listing.objectValues()
        self.assertEquals('File', doc.portal_type)
        self.assertEquals('world.txt', doc.Title())
        if IS_PLONE_5:
            self.assertEquals('Hello World', doc.file.data.strip())
        else:
            self.assertEquals('Hello World', doc.getFile().data.strip())

    @browsing
    def test_create_ftw_files_in_filelistingblock(self, browser):
        """
        Ensure filelistingblock uploads can be configured to create
        ftw.file.File
        """
        fti = getUtility(IDexterityFTI,
                         name='ftw.simplelayout.FileListingBlock')
        act = list(fti.allowed_content_types)
        act.insert(0, 'ftw.file.File')
        fti.allowed_content_types = tuple(act)

        listing = create(Builder('sl listingblock').titled(u'Downloads')
                         .within(create(Builder('sl content page')
                                 .titled(u'Page'))))
        self.assertEquals([], listing.objectIds())

        browser.login()
        self.make_dropzone_upload(listing, self.asset('world.txt').open('rb'))
        self.assertEqual(201, browser.status_code)
        self.assertEqual(
            {u'content': u'Created',
             u'url': u'http://nohost/plone/page/downloads/world.txt',
             u'proceed': True},
            browser.json
        )
        self.assertEquals(['world.txt'], listing.objectIds())

        doc, = listing.objectValues()
        self.assertEquals('ftw.file.File', doc.portal_type)

    @browsing
    def test_create_images_in_galleries(self, browser):
        gallery = create(Builder('sl galleryblock').titled(u'Gallery')
                         .within(create(Builder('sl content page').titled(u'Page'))))
        self.assertEquals([], gallery.objectIds())

        browser.login()
        self.make_dropzone_upload(gallery, self.asset('cropped.jpg').open('rb'))
        self.assertEqual(201, browser.status_code)
        self.assertEqual({u'content': u'Created',
                          u'url': u'http://nohost/plone/page/gallery/cropped.jpg',
                          u'proceed': True}, browser.json)
        self.assertEquals(['cropped.jpg'], gallery.objectIds())

        img, = gallery.objectValues()
        self.assertEquals('Image', img.portal_type)
        self.assertEquals('cropped.jpg', img.Title())

    @browsing
    def test_files_are_not_allowed_in_galleries(self, browser):
        gallery = create(Builder('sl galleryblock').titled(u'Gallery')
                         .within(create(Builder('sl content page').titled(u'Page'))))
        self.assertEquals([], gallery.objectIds())

        with browser.login().expect_http_error(400):
            self.make_dropzone_upload(gallery, self.asset('world.txt').open('rb'))

        self.assertEqual({u'error': u'Only images can be added to the gallery.',
                          u'proceed': False}, browser.json)
        self.assertEquals([], gallery.objectIds())

    def make_dropzone_upload(self, context, file_):
        """The dropzone JS makes a multipart upload with the the file as field named "file".
        The testbrowser does not support making multipart requests directly (we would have to
        create the MIME body ourself), but the form filling does that well.
        Therefore we use an HTML form for uploading.
        """
        html = (
            '<form action="{}/@@dropzone-upload" method="post" enctype="multipart/form-data">'
            '  <input type="file" name="file" />'
            '  <input type="hidden" name="_authenticator" value="{}" />'
            '  <input type="submit" />'
            '</form>').format(context.absolute_url(), createToken())

        defaultbrowser.open(context).parse(html)
        defaultbrowser.fill({'file': file_}).submit()
        return defaultbrowser



from ftw.simplelayout.utils import IS_PLONE_5
from ftw.upgrade import UpgradeStep
from plone import api


class MoveJSToRegistryForMapblockForPlone5(UpgradeStep):
    """Move JS to registry for mapblock. Only for plone 5
    """

    def __call__(self):

        if IS_PLONE_5:
            self.install_upgrade_profile()

            record = 'plone.bundles/plone-legacy.resources'
            resources = api.portal.get_registry_record(record)
            name = u'resource-ftw-simplelayout-videoblock-js'
            if name in resources:
                resources.remove(name)



from ftw.simplelayout import _
from ftw.simplelayout.images.interfaces import IImageLimitValidatorMessages
from ftw.simplelayout.images.interfaces import IImageLimits
from ftw.simplelayout.images.limits.limits import Limits
from z3c.form import validator
from z3c.form.interfaces import NOT_CHANGED
from zope.interface import Invalid
from zope.interface import implementer


class LimitValidatorMessages(object):
    def __init__(self):
        self.limits = Limits()

    def limit_not_satisfied_message(self, limit_type, identifier, image):
        if limit_type == 'soft':
            return self._soft_limit_not_satisfied_message(identifier, image)
        elif limit_type == 'hard':
            return self._hard_limit_not_satisfied_message(identifier, image)

        return ''

    def limit_str(self, limit_type, identifier, image):
        limits = self._get_image_limits_for(limit_type, identifier)

        current_width = image._width
        current_height = image._height

        width = limits.get('width', 0)
        height = limits.get('height', 0)

        width_str = self._width_str(width, current_width)
        height_str = self._height_str(height, current_height)

        limit_str = ''
        if width and height:
            limit_str = _(u'limit_width_and_height',
                          default=u"${width_str} and ${height_str}",
                          mapping={'width_str': width_str, 'height_str': height_str})
        elif width:
            limit_str = width_str

        elif height:
            limit_str = height_str

        return limit_str

    def _hard_limit_not_satisfied_message(self, identifier, image):
        limit_str = self.limit_str('hard', identifier, image)

        return _(u'hard_limit_not_satisfied',
                 default=u"The image doesn't fit the required dimensions of ${limit_str}",
                 mapping={'limit_str': limit_str})

    def _soft_limit_not_satisfied_message(self, identifier, image):
        limit_str = self.limit_str('soft', identifier, image)

        return _(u'soft_limit_not_satisfied',
                 default=u"Optimal image quality: ${limit_str}",
                 mapping={'limit_str': limit_str})

    def _width_str(self, width, current_width):
        return _(u'limit_width_str',
                 default=u"width: ${width}px (current: ${current_width}px)",
                 mapping={'width': width, 'current_width': current_width}
                 )

    def _height_str(self, height, current_height):
        return _(u'limit_height_str',
                 default=u"height: ${height}px (current: ${current_height}px)",
                 mapping={'height': height, 'current_height': current_height}
                 )

    def _get_image_limits_for(self, limit_type, identifier):
        return self.limits.get_limits_for(limit_type, identifier)


@implementer(IImageLimitValidatorMessages)
class ImageLimitValidatorMessages(object):
    def __init__(self, context):
        self.context = context
        self.limit_validator_messages = LimitValidatorMessages()
        self.image_limits = IImageLimits(self.context)

    def limit_not_satisfied_message(self, limit_type):
        return self.limit_validator_messages.limit_not_satisfied_message(
            limit_type, self.image_limits.identifier, self.image_limits._image)

    def limit_str(self, limit_type):
        return self.limit_validator_messages.limit_str(
            limit_type, self.image_limits.identifier, self.image_limits._image)


class ImageLimitValidator(validator.SimpleFieldValidator):
    """Validates the image-dimensions.

    Soft-limit: Validates if the given image has at least the dimensions given
    within the ImageLimit-configuration for the current contenttype.

    Hard-limit: Validates if the given image has at least the dimensions given
    within the ImageLimit-configuration for the current contenttype.
    """

    # The identifier should be the name of your contenttype. Because we
    # don't know the contenttype within a SimpleFieldValidator on an add-form,
    # we have to define it here explicitly.
    identifier = None

    def __init__(self, *args, **kwargs):
        super(ImageLimitValidator, self).__init__(*args, **kwargs)
        self.limits = Limits()
        self.validator_messages = LimitValidatorMessages()

    def validate(self, value):
        super(ImageLimitValidator, self).validate(value)
        if not value:
            return

        self._validate_hard_limit(value)

    def _validate_hard_limit(self, value):
        if value == NOT_CHANGED:
            value = getattr(self.context, self.field.getName())
        if not self._validate_limit_for('hard', value):
            raise Invalid(self.validator_messages.limit_not_satisfied_message(
                'hard', self.identifier, value))

    def _validate_limit_for(self, limit_type, value):
        return self.limits.validate(
            limit_type, self.identifier, width=value._width, height=value._height)



from zope.interface import implements
from plone.app.portlets.portlets import base
from plone.portlets.interfaces import IPortletDataProvider
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone import api
from ftw.simplelayout.interfaces import IPageConfiguration


class ISimplelayoutPortlet(IPortletDataProvider):
    """
    Marker Interface for the simplelayout portlet.
    """


class Assignment(base.Assignment):

    implements(ISimplelayoutPortlet)

    @property
    def title(self):
        return "Simplelayout Portlet"


class Renderer(base.Renderer):

    render = ViewPageTemplateFile('templates/portlet.pt')

    @property
    def available(self):
        if not IPageConfiguration(self.context, None):
            return False
        if self.has_blocks():
            return True
        else:
            disabled = int(self.request.get('disable_border', 0)) == 1
            return api.user.has_permission('Modify portal content',
                                           obj=self.context) and not disabled

    def has_blocks(self):
        config = IPageConfiguration(self.context)
        data = config.load()

        portlet_container = []
        if self.manager.__name__ == 'plone.rightcolumn':
            portlet_container = data.get('portletright', [])
        elif self.manager.__name__ == 'plone.leftcolumn':
            portlet_container = data.get('portletleft', [])

        for layout in portlet_container:
            for columns in layout.values():
                for column in columns:
                    if len(column['blocks']):
                        return True
                    else:
                        continue
        return False


class AddForm(base.NullAddForm):
    def create(self):
        return Assignment()



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.interfaces import ISimplelayoutDefaultSettings
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.simplelayout.utils import IS_PLONE_5
from ftw.testbrowser import browsing
from plone import api
from plone.app.textfield.value import RichTextValue
from plone.namedfile.file import NamedBlobImage
from plone.registry.interfaces import IRegistry
from plone.uuid.interfaces import IUUID
from StringIO import StringIO
from unittest import TestCase
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.component import queryUtility
from zope.intid.interfaces import IIntIds
import json
import transaction


if IS_PLONE_5:
    from Products.CMFPlone.interfaces.controlpanel import IImagingSchema


class TestTextBlockRendering(TestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def setUp(self):
        self.page = create(Builder('sl content page'))

        self.image = StringIO(
            'GIF89a\x01\x00\x01\x00\x80\x00\x00\x00\x00\x00\x00\x00'
            '\x00!\xf9\x04\x04\x00\x00\x00\x00,\x00\x00\x00\x00\x01\x00'
            '\x01\x00\x00\x02\x02D\x01\x00;')

    @browsing
    def test_teaser_url_external_on_image_and_title(self, browser):
        block = create(Builder('sl textblock')
                       .within(self.page)
                       .titled('TextBlock title')
                       .having(text=RichTextValue('The text'))
                       .having(external_link='http://www.4teamwork.ch')
                       .having(image=NamedBlobImage(data=self.image.read(),
                                                    filename=u'test.gif'))
                       .having(open_image_in_overlay=True))

        browser.login().visit(block, view='@@block_view')
        self.assertEquals(
            'http://www.4teamwork.ch',
            browser.css('h2 a').first.attrib['href'])

        self.assertEquals(
            'http://www.4teamwork.ch',
            browser.css('.sl-image a').first.attrib['href'])

    @browsing
    def test_teaser_url_internal(self, browser):
        intids = getUtility(IIntIds)
        block = create(Builder('sl textblock')
                       .within(self.page)
                       .titled('TextBlock title')
                       .having(text=RichTextValue('The text'))
                       .having(internal_link=RelationValue(
                               intids.getId(self.page)))
                       .having(image=NamedBlobImage(data=self.image.read(),
                                                    filename=u'test.gif'))
                       .having(open_image_in_overlay=True))

        browser.login().visit(block, view='@@block_view')

        self.assertEquals(
            self.page.absolute_url(),
            browser.css('h2 a').first.attrib['href'])

        self.assertEquals(
            self.page.absolute_url(),
            browser.css('.sl-image a').first.attrib['href'])

    @browsing
    def test_init_scale_is_first_from_block_actions(self, browser):
        block = create(Builder('sl textblock')
                       .within(self.page)
                       .titled('TextBlock title')
                       .having(text=RichTextValue('The text'))
                       .having(image=NamedBlobImage(data=self.image.read(),
                                                    filename=u'test.gif')))

        browser.login().visit(block, view='@@block_view')
        self.assertEquals('sl-image sl_textblock_small left',
                          browser.css('.sl-image').first.attrib['class'])

    @browsing
    def test_change_image_scale(self, browser):
        block = create(Builder('sl textblock')
                       .within(self.page)
                       .titled('TextBlock title')
                       .having(text=RichTextValue('The text'))
                       .having(image=NamedBlobImage(data=self.image.read(),
                                                    filename=u'test.gif')))

        payload = {'data': json.dumps({'uid': IUUID(block), 'scale': 'large'})}
        browser.login().visit(self.page,
                              view='sl-ajax-reload-block-view',
                              data=payload)
        browser.visit(block, view='@@block_view')

        self.assertEquals('sl-image large left',
                          browser.css('.sl-image').first.attrib['class'])

    @browsing
    def test_textblock_title_not_rendered_when_empty(self, browser):
        """
        This test makes sure that the title of the block is only rendered
        if there is a title. Otherwise we'll end up with an empty HTML
        tag in the template.
        """
        textblock = create(Builder('sl textblock')
                           .titled('My textblock')
                           .having(show_title=True)
                           .within(self.page))

        browser.login().visit(self.page)

        title_css_selector = '.ftw-simplelayout-textblock h2'

        # Make sure the title is here (in its tag).
        self.assertEqual('My textblock',
                         browser.css(title_css_selector).first.text)

        # Remove the title of the block and make sure the tag is no longer
        # there.
        textblock.title = ''
        transaction.commit()
        browser.login().visit(self.page)
        self.assertEqual([], browser.css(title_css_selector))

    @browsing
    def test_image_alt_text_is_rendered(self, browser):
        """
        This test makes sure that the image's alt text is rendered in the
        template.
        """
        alt_text = u'A very nice im\xe4ge'
        block = create(Builder('sl textblock')
                       .within(self.page)
                       .titled('TextBlock title')
                       .having(text=RichTextValue('The text'))
                       .having(image=NamedBlobImage(data=self.image.read(),
                                                    filename=u'test.gif'))
                       .having(image_alt_text=alt_text))

        payload = {'data': json.dumps({'uid': IUUID(block), 'scale': 'large'})}
        browser.login().visit(self.page,
                              view='sl-ajax-reload-block-view',
                              data=payload)
        browser.visit(self.page)

        self.assertEquals(
            alt_text,
            browser.css('.ftw-simplelayout-textblock img').first.attrib['alt']
        )

    @browsing
    def test_image_alt_text_empty(self, browser):
        """
        This test makes sure that there is an alt property on the img tag
        even if no alternative text has been entered.
        """
        block = create(Builder('sl textblock')
                       .within(self.page)
                       .titled('TextBlock title')
                       .having(text=RichTextValue('The text'))
                       .having(image=NamedBlobImage(data=self.image.read(),
                                                    filename=u'test.gif')))

        payload = {'data': json.dumps({'uid': IUUID(block), 'scale': 'large'})}
        browser.login().visit(self.page,
                              view='sl-ajax-reload-block-view',
                              data=payload)
        browser.visit(block, view='@@block_view')

        self.assertEquals('', browser.css('img').first.attrib['alt'])

    @browsing
    def test_image_overlay(self, browser):
        """
        This test makes sure that the link to the overlay contains
        the url to the image to be displayed in the overlay.
        """
        block = create(Builder('sl textblock')
                       .within(self.page)
                       .titled('TextBlock title')
                       .having(text=RichTextValue('The text'))
                       .having(image=NamedBlobImage(data=self.image.read(),
                                                    filename=u'test.gif'))
                       .having(open_image_in_overlay=True,
                               ))

        browser.login().visit(block, view='@@block_view')
        link_url = browser.css('a.colorboxLink').first.attrib['href']

        browser.open(link_url)

        if IS_PLONE_5:
            self.assertEquals('image/png', browser.headers['content-type'])
        else:
            self.assertEquals('image/jpeg', browser.headers['content-type'])

    @browsing
    def test_image_overlay_when_scale_is_missing(self, browser):
        if IS_PLONE_5:
            registry = queryUtility(IRegistry)
            sizes = registry.forInterface(IImagingSchema, prefix="plone").allowed_sizes

        else:
            ptool = api.portal.get_tool('portal_properties')
            sizes = ptool.get('imaging_properties').allowed_sizes

        # Remove the colorbox scale.
        allowed_sizes = filter(lambda x: not x.startswith('colorbox'), sizes)

        if IS_PLONE_5:
            registry.forInterface(IImagingSchema, prefix="plone").allowed_sizes = allowed_sizes
        else:
            ptool.get('imaging_properties').allowed_sizes = tuple(allowed_sizes)
        transaction.commit()

        block = create(Builder('sl textblock')
                       .within(self.page)
                       .titled('TextBlock title')
                       .having(text=RichTextValue('The text'))
                       .having(image=NamedBlobImage(data=self.image.read(),
                                                    filename=u'test.gif'))
                       .having(open_image_in_overlay=True))
        browser.login().visit(block, view='@@block_view')

        # There must be no link to the overlay since we were unable to
        # determine the url to the overlay image.
        self.assertEquals(
            [],
            browser.css('a.colorboxLink')
        )

    @browsing
    def test_auto_appending_enlarged_picture_for_overlay(self, browser):
        block = create(Builder('sl textblock')
                       .within(self.page)
                       .titled('TextBlock title')
                       .having(text=RichTextValue('The text'))
                       .having(image=NamedBlobImage(data=self.image.read(),
                                                    filename=u'test.gif'))
                       .having(open_image_in_overlay=True,
                               image_alt_text='Some alt text'))

        browser.login().visit(block, view='@@block_view')
        self.assertTrue(
            browser.css('a.colorboxLink').first.attrib['title'].endswith(
                'enlarged picture.'),
            'Expect "enlarged picture." hint in alt text of img.')

    @browsing
    def test_title_only_css_class(self, browser):
        create(Builder('sl textblock')
               .within(self.page)
               .titled('TextBlock title'))

        browser.login().visit(self.page)
        self.assertTrue(
            browser.css('.sl-block.titleOnly'), 'Expext title only class.')

    @browsing
    def test_data_caption_holds_caption_of_image(self, browser):
        """
        This test makes sure that there is an attribute "data-caption"
        on the link opening the colorbox and that it holds the caption
        defined on the textblock.
        """
        block = create(Builder('sl textblock')
                       .within(self.page)
                       .titled('TextBlock title')
                       .having(image_caption=u'The caption')
                       .having(text=RichTextValue('The text'))
                       .having(image=NamedBlobImage(data=self.image.read(),
                                                    filename=u'test.gif'))
                       .having(open_image_in_overlay=True,))

        browser.login().visit(block, view='@@block_view')
        self.assertEqual(
            u'The caption',
            browser.css('a.colorboxLink').first.attrib['data-caption']
        )

    def set_config(self, config={}):
        api.portal.set_registry_record(
            'image_limits', config, ISimplelayoutDefaultSettings)

        transaction.commit()

    @browsing
    def test_show_soft_limit_indicator_if_soft_limit_is_not_satisfied(self, browser):
        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock').within(page).with_dummy_image())

        browser.login().visit(block)
        self.assertEquals(0, len(browser.css('.softLimitIndicator')))

        self.set_config({
            block.portal_type: [
                u'soft: width={}'.format(block.image._width + 100)
            ]}
        )

        browser.visit(block)

        self.assertEquals(1, len(browser.css('.softLimitIndicator')))

    @browsing
    def test_show_hard_limit_indicator_if_hard_limit_is_not_satisfied(self, browser):
        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock').within(page).with_dummy_image())

        browser.login().visit(block)
        self.assertEquals(0, len(browser.css('.hardLimitIndicator')))

        self.set_config({
            block.portal_type: [
                u'hard: width={}'.format(block.image._width + 100)
            ]}
        )

        browser.visit(block)

        self.assertEquals(1, len(browser.css('.hardLimitIndicator')))

    @browsing
    def test_show_only_hard_limit_indicator_if_hard_and_soft_limit_are_not_satisfied(self, browser):
        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock').within(page).with_dummy_image())

        browser.login().visit(block)
        self.assertEquals(0, len(browser.css('.limitIndicator')))

        self.set_config({
            block.portal_type: [
                u'soft: width={}'.format(block.image._width + 200),
                u'hard: width={}'.format(block.image._width + 100)
            ]}
        )

        browser.visit(block)

        self.assertEquals(1, len(browser.css('.limitIndicator')))
        self.assertEquals(1, len(browser.css('.hardLimitIndicator')))

    @browsing
    def test_do_not_show_limit_indicator_if_all_limits_are_satisfied(self, browser):
        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock').within(page).with_dummy_image())

        browser.login().visit(block)
        self.assertEquals(0, len(browser.css('.limitIndicator')))

        self.set_config({
            block.portal_type: [
                u'soft: width={}'.format(block.image._width - 100),
                u'hard: width={}'.format(block.image._width - 200)
            ]}
        )

        browser.visit(block)
        self.assertEquals(0, len(browser.css('.limitIndicator')))

    @browsing
    def test_only_show_limit_indicator_for_editors(self, browser):
        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock').within(page).with_dummy_image())

        self.set_config({
            block.portal_type: [
                u'soft: width={}'.format(block.image._width + 100)
            ]}
        )

        browser.login().visit(block)
        self.assertEquals(1, len(browser.css('.limitIndicator')))

        browser.logout().visit(block)
        self.assertEquals(0, len(browser.css('.limitIndicator')))

    @browsing
    def test_limit_indicator_respects_cropped_image(self, browser):
        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock').within(page)
                       .with_dummy_image()
                       .with_cropped_image())

        image_limit = block.cropped_image._width + 100

        self.set_config({
            block.portal_type: [
                u'soft: width={}'.format(image_limit)
            ]}
        )

        # This only verifies the image widths for further assertions.
        # The image should be higher thant the limit and the cropped image
        # should be lower than the limit.
        self.assertGreater(block.image._width, image_limit)
        self.assertLess(block.cropped_image._width, image_limit)

        browser.login().visit(block)
        self.assertEquals(1, len(browser.css('.limitIndicator')))

    @browsing
    def test_display_corpped_image_if_available(self, browser):
        # Do not commit the transaction in this test. Otherwise the test will
        # always pass because the scaled image will be recreated even if it's
        # the same image as before the transaction.

        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock').within(page)
                       .with_dummy_image()
                       .with_cropped_image())

        block_view = block.restrictedTraverse('block_view')
        browser.open_html(block_view())

        url = browser.css('.sl-image img').first.get('src')

        block.cropped_image = None

        browser.open_html(block_view())
        self.assertNotEqual(browser.css('.sl-image img').first.get('src'), url)

    @browsing
    def test_show_cropped_image_in_overlay_if_attribute_is_set(self, browser):
        # Do not commit the transaction in this test. Otherwise the test will
        # always pass because the scaled image will be recreated even if it's
        # the same image as before the transaction.

        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock').within(page)
                       .with_dummy_image()
                       .with_cropped_image()
                       .having(
                           open_image_in_overlay=True,
                           use_cropped_image_for_overlay=False))

        block_view = block.restrictedTraverse('block_view')
        browser.open_html(block_view())

        url = browser.css('.sl-image .colorboxLink').first.get('href')

        block.use_cropped_image_for_overlay = True

        browser.open_html(block_view())

        self.assertNotEqual(browser.css('.sl-image .colorboxLink').first.get('href'), url)

    @browsing
    def test_if_harmeful_css_classes_are_removed(self, browser):
        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock')
                       .having(text=RichTextValue(
                           '<p id="test-textblock" class="anotherclass sl-simplelayout sl-layout">The text</p>'))
                       .within(page))

        browser.login().visit(page)
        self.assertNotIn('sl-simplelayout', browser.css('#test-textblock').first.attrib['class'])
        self.assertNotIn('sl-layout', browser.css('#test-textblock').first.attrib['class'])



from Products.Five.browser import BrowserView
from plone.app.uuid.utils import uuidToObject
from zExceptions import BadRequest
import json


class CropImageRedirector(BrowserView):

    def __call__(self):
        payload = self.request.get('data', None)
        if not payload:
            raise BadRequest('No data given')

        data = json.loads(payload)
        block = uuidToObject(data['block'])

        return self.request.RESPONSE.redirect('{0}/image_cropping.json'.format(
            block.absolute_url()))



from ftw.upgrade import UpgradeStep


class MoveOptionToHideABlockToABehavior(UpgradeStep):
    """Move option to hide a block to a behavior.
    """

    def __call__(self):
        self.install_upgrade_profile()
